@@ -46,6 +46,7 @@ typedef struct busy_cmd {
 	char *options;
 	cmd_handler_t handler;
 	int ops_num;
+	char *info;
 } busy_cmd_t;
 
 
@@ -59,6 +60,7 @@ extern errno_t create_cmd(busy_ctx_t *ctx);
 extern errno_t ln_cmd(busy_ctx_t *ctx);
 extern errno_t rm_cmd(busy_ctx_t *ctx);
 extern errno_t cp_cmd(busy_ctx_t *ctx);
+extern errno_t trunc_cmd(busy_ctx_t *ctx);
 
 
 extern reiser4_object_t *busy_misc_open_parent(reiser4_tree_t *tree, 
