@@ -10,11 +10,12 @@
 
 #include <reiser4/types.h>
 
-extern void reiser4_print_format(reiser4_format_t *format,
-				 uint16_t options);
+extern void reiser4_print_format(reiser4_format_t *format, uint16_t options)
+				__attribute__((used));
 
 extern void reiser4_print_node(reiser4_node_t *node, uint32_t start, 
-			       uint32_t count, uint16_t options);
+			       uint32_t count, uint16_t options)
+			       __attribute__((used));
 #endif
 
 #endif
