@@ -8,7 +8,7 @@
 
 #include <reiser4/types.h>
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 extern void reiser4_key_free(reiser4_key_t *key);
 
 extern void reiser4_key_print(reiser4_key_t *key,
@@ -46,12 +46,12 @@ extern errno_t reiser4_key_build_generic(reiser4_key_t *key,
 					 oid_t objectid,
 					 uint64_t offset);
 
-extern errno_t reiser4_key_build_hashed(reiser4_key_t *key,
-					reiser4_plug_t *hash,
-					reiser4_plug_t *fibre,
-					oid_t locality,
-					oid_t objectid,
-					char *name);
+extern void reiser4_key_build_hashed(reiser4_key_t *key,
+				     reiser4_plug_t *hash,
+				     reiser4_plug_t *fibre,
+				     oid_t locality,
+				     oid_t objectid,
+				     char *name);
 
 extern void reiser4_key_minimal(reiser4_key_t *key);
 #endif
