@@ -8,7 +8,7 @@
 
 #include <reiser4/types.h>
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 extern errno_t reiser4_node_sync(reiser4_node_t *node);
 extern uint16_t reiser4_node_space(reiser4_node_t *node);
 extern uint16_t reiser4_node_overhead(reiser4_node_t *node);
@@ -19,15 +19,15 @@ extern uint64_t reiser4_node_get_fstamp(reiser4_node_t *node);
 extern errno_t reiser4_node_update_ptr(reiser4_node_t *node);
 extern void reiser4_node_move(reiser4_node_t *node, blk_t nr);
 
-extern errno_t callback_node_insert(reiser4_node_t *node, pos_t *pos,
-				    trans_hint_t *hint);
+extern errno_t cb_node_insert(reiser4_node_t *node, pos_t *pos,
+			      trans_hint_t *hint);
 
-extern errno_t callback_node_write(reiser4_node_t *node, pos_t *pos,
-				   trans_hint_t *hint);
+extern errno_t cb_node_write(reiser4_node_t *node, pos_t *pos,
+			     trans_hint_t *hint);
 
 extern reiser4_node_t *reiser4_node_create(reiser4_tree_t *tree,
-				   blk_t nr, rid_t pid,
-				   uint8_t level);
+					   reiser4_plug_t *plug,
+					   blk_t nr, uint8_t level);
 
 extern errno_t reiser4_node_trav(reiser4_node_t *node,
 				 place_func_t func,
@@ -84,18 +84,16 @@ extern lookup_t reiser4_node_lookup(reiser4_node_t *node,
 				    lookup_bias_t bias,
 				    pos_t *pos);
 
-extern errno_t reiser4_node_fini(reiser4_node_t *node);
 extern errno_t reiser4_node_close(reiser4_node_t *node);
 extern uint32_t reiser4_node_items(reiser4_node_t *node);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
+extern errno_t reiser4_node_fini(reiser4_node_t *node);
 extern bool_t reiser4_node_isdirty(reiser4_node_t *node);
 extern void reiser4_node_mkdirty(reiser4_node_t *node);
 extern void reiser4_node_mkclean(reiser4_node_t *node);
 #endif
 
-#define node_blocknr(node) ((node)->block->nr)
-
 extern void reiser4_node_lock(reiser4_node_t *node);
 extern void reiser4_node_unlock(reiser4_node_t *node);
 extern bool_t reiser4_node_locked(reiser4_node_t *node);
