@@ -6,6 +6,9 @@
 #ifndef REPAIR_ALLOC_H
 #define REPAIR_ALLOC_H
 
+extern errno_t repair_alloc_check_struct(reiser4_alloc_t *alloc, 
+					 uint8_t mode);
+
 extern errno_t repair_alloc_layout_bad(reiser4_alloc_t *alloc, 
 				       region_func_t func, void *data);
 
@@ -17,4 +20,6 @@ extern reiser4_alloc_t *repair_alloc_unpack(reiser4_fs_t *fs,
 
 extern void repair_alloc_print(reiser4_alloc_t *alloc, aal_stream_t *stream);
 
+extern errno_t repair_alloc_open(reiser4_fs_t *fs, uint8_t mode);
+
 #endif
