@@ -16,8 +16,6 @@ extern errno_t repair_fs_open(repair_data_t *repair,
 
 extern errno_t repair_fs_replay(reiser4_fs_t *fs);
 
-extern void repair_fs_close(reiser4_fs_t *fs);
-
 extern errno_t repair_fs_pack(reiser4_fs_t *fs, 
 			      aux_bitmap_t *bitmap, 
 			      aal_stream_t *stream);
@@ -29,4 +27,13 @@ extern reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 extern errno_t repair_fs_lost_key(reiser4_fs_t *fs, 
 				  reiser4_key_t *key);
 
+extern errno_t repair_fs_check_backup(aal_device_t *device, 
+				      backup_hint_t *hint);
+
+extern errno_t repair_fs_block_pack(aal_block_t *block, 
+				    aal_stream_t *stream);
+
+extern aal_block_t *repair_fs_block_unpack(reiser4_fs_t *fs, 
+					   aal_stream_t *stream);
+
 #endif
