@@ -20,7 +20,11 @@ int64_t reiser4_flow_read(reiser4_tree_t *tree, trans_hint_t *hint) {
 	
 	buff = hint->specific;
 	reiser4_key_assign(&key, &hint->offset);
-	
+
+#ifndef ENABLE_MINIMAL
+	hint->blocks = tree->blocks;
+#endif
+
 	for (total = 0, size = hint->count; size > 0; ) {
 		int32_t read;
 		reiser4_place_t place;
@@ -78,10 +82,6 @@ int64_t reiser4_flow_read(reiser4_tree_t *tree, trans_hint_t *hint) {
 			/* Prepare hint for read */
 			hint->count = size;
 
-#ifndef ENABLE_MINIMAL
-			hint->blocks = tree->blocks;
-#endif
-			
 			/* Read data from the tree */
 			if ((read = plug_call(place.plug->o.item_ops->object,
 					      read_units, &place, hint)) < 0) 
@@ -110,9 +110,7 @@ int64_t reiser4_flow_read(reiser4_tree_t *tree, trans_hint_t *hint) {
 #ifndef ENABLE_MINIMAL
 /* Releases passed region in block allocator. This is used in tail during tree
    trunacte. */
-static errno_t cb_release_region(void *entity, uint64_t start,
-				 uint64_t width, void *data)
-{
+static errno_t cb_release_region(uint64_t start, uint64_t width, void *data) {
 	reiser4_tree_t *tree = (reiser4_tree_t *)data;
 	return reiser4_alloc_release(tree->fs->alloc, start, width);
 }
@@ -133,6 +131,8 @@ int64_t reiser4_flow_write(reiser4_tree_t *tree, trans_hint_t *hint) {
 	buff = hint->specific;
 	reiser4_key_assign(&key, &hint->offset);
 
+	hint->blocks = tree->blocks;
+
 	/* Loop until desired number of bytes is written. */
 	for (total = bytes = 0, size = hint->count; size > 0;) {
 		int32_t write;
@@ -157,6 +157,7 @@ int64_t reiser4_flow_write(reiser4_tree_t *tree, trans_hint_t *hint) {
 
 		/* level new item will be inserted a on. */
 		level = reiser4_tree_target_level(tree, hint->plug);
+		hint->bytes = 0;
 
 		/* Writing data to tree. */
 		if ((write = reiser4_tree_write(tree, &place,
@@ -193,6 +194,7 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 	errno_t res;
 	int64_t trunc;
 	uint32_t size;
+	uint64_t bytes;
 	uint64_t total;
 	reiser4_key_t key;
 
@@ -204,8 +206,9 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 	/* Setting up region func to release region callback. It is needed for
 	   releasing extent blocks. */
 	hint->region_func = cb_release_region;
+	hint->blocks = tree->blocks;
 
-	for (total = 0, size = hint->count; size > 0;
+	for (total = bytes = 0, size = hint->count; size > 0;
 	     size -= trunc, total += trunc)
 	{
 		lookup_hint_t lhint;
@@ -255,6 +258,7 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 		}
 
 		hint->count = size;
+		hint->bytes = 0;
 
 		/* Calling node truncate method. */
 		if ((trunc = reiser4_node_trunc(place.node, &place.pos,
@@ -263,6 +267,8 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 			return trunc;
 		}
 		
+		bytes += hint->bytes;
+		
 		/* Updating left delimiting keys in all parent nodes */
 		if (reiser4_place_leftmost(&place) &&
 		    place.node->p.node)
@@ -310,6 +316,8 @@ int64_t reiser4_flow_truncate(reiser4_tree_t *tree, trans_hint_t *hint) {
 
 		reiser4_key_inc_offset(&hint->offset, trunc);
 	}
+	
+	hint->bytes = bytes;
 
 	reiser4_key_assign(&hint->offset, &key);
 	return total;
@@ -341,8 +349,8 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 	/* Check if number of bytes to be converted is not multiple of block
 	   size and this is tail2extent conversion. If so, have to align
 	   @hint->count byblock size into highest side. */
-	if (hint->plug->id.group == EXTENT_ITEM &&
-	    (hint->count & (blksize - 1)) != 0)
+	if (hint->plug->id.group == EXTENT_ITEM && 
+	    hint->count != MAX_UINT64 && (hint->count & (blksize - 1)) != 0)
 	{
 		hint->count += blksize -
 			(hint->count & (blksize - 1));
@@ -386,7 +394,11 @@ errno_t reiser4_flow_convert(reiser4_tree_t *tree, conv_hint_t *hint) {
 		trans.data = tree;
 		
 		if (conv == 0) {
-			/* If nothing was read, the hole will be inserted. */
+			/* If nothing was read, depending on hint->ins_hole flag
+			   the hole will be inserted or convertion is finished. */
+			if (!hint->ins_hole)
+				break;
+
 			trans.specific = NULL;
 		} else {
 			/* Trunc & insert only read @count bytes. */
