@@ -6,7 +6,7 @@
 
 #include <reiser4/libreiser4.h>
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 bool_t reiser4_node_isdirty(reiser4_node_t *node) {
 	aal_assert("umka-2663", node != NULL);
 	return node->block->dirty;
@@ -24,28 +24,24 @@ void reiser4_node_mkclean(reiser4_node_t *node) {
 
 /* Creates new node at block @nr on @tree with @level and with plugin @pid. Uses
    tree instance for accessing block size and key plugin in use. */
-reiser4_node_t *reiser4_node_create(reiser4_tree_t *tree, blk_t nr,
-				    rid_t pid, uint8_t level)
+reiser4_node_t *reiser4_node_create(reiser4_tree_t *tree, 
+				    reiser4_plug_t *plug,
+				    blk_t nr, uint8_t level)
 {
 	uint32_t size;
 	aal_block_t *block;
 	reiser4_node_t *node;
-	reiser4_plug_t *plug;
 	aal_device_t *device;
 
 	aal_assert("umka-1268", tree != NULL);
+	aal_assert("vpf-1596", plug != NULL);
+	aal_assert("vpf-1654", tree->fs != NULL);
+	aal_assert("vpf-1655", tree->fs->device != NULL);
     
-	/* Finding the node plugin by its id */
-	if (!(plug = reiser4_factory_ifind(NODE_PLUG_TYPE, pid))) {
-		aal_error("Can't find node plugin by its id 0x%x.",
-			  pid);
-		return NULL;
-	}
-
 	/* Getting tree tree device and blksize in use to use them for creating
 	   new node. */
 	size = reiser4_tree_get_blksize(tree);
-	device = reiser4_tree_get_device(tree);
+	device = tree->fs->device;
 
 	/* Allocate new node of @size at @nr. */
 	if (!(block = aal_block_alloc(device, size, nr)))
@@ -81,7 +77,7 @@ errno_t reiser4_node_trav(reiser4_node_t *node, place_func_t func, void *data) {
 		if ((res = reiser4_place_open(&place, node, pos))) {
 			aal_error("Node (%llu), item (%u): failed to "
 				  "open the item by its place.", 
-				  node_blocknr(node), pos->item);
+				  node->block->nr, pos->item);
 			return res;
 		}
 		
@@ -114,7 +110,7 @@ bool_t reiser4_node_locked(reiser4_node_t *node) {
 	return node->counter > 0 ? 1 : 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Assigns @nr block number to @node. */
 void reiser4_node_move(reiser4_node_t *node, blk_t nr) {
 	aal_assert("umka-2248", node != NULL);
@@ -135,10 +131,12 @@ reiser4_node_t *reiser4_node_open(reiser4_tree_t *tree, blk_t nr) {
         reiser4_node_t *node;
  
         aal_assert("umka-160", tree != NULL);
+        aal_assert("vpf-1652", tree->fs != NULL);
+        aal_assert("vpf-1653", tree->fs->device != NULL);
 
 	/* Getting tree characteristics needed for open node. */
 	size = reiser4_tree_get_blksize(tree);
-	device = reiser4_tree_get_device(tree);
+	device = tree->fs->device;
 	
 	/* Load block at @nr, that node lie in. */
 	if (!(block = aal_block_load(device, size, nr))) {
@@ -170,17 +168,17 @@ reiser4_node_t *reiser4_node_open(reiser4_tree_t *tree, blk_t nr) {
         return NULL;
 }
 
+#ifndef ENABLE_MINIMAL
 /* Saves node to device if it is dirty and closes node */
 errno_t reiser4_node_fini(reiser4_node_t *node) {
-#ifndef ENABLE_STAND_ALONE
 	/* Node should be clean when it is going to be closed. */
 	if (reiser4_node_isdirty(node) && reiser4_node_sync(node)) {
-		aal_error("Can't write node %llu.", node_blocknr(node));
+		aal_error("Can't write node %llu.", node->block->nr);
 	}
-#endif
 
 	return reiser4_node_close(node);
 }
+#endif
 
 /* Closes specified node and its children. Before the closing, this function
    also detaches nodes from the tree if they were attached. */
@@ -271,6 +269,9 @@ lookup_t reiser4_node_lookup(reiser4_node_t *node,
 			pos->item++;
 			return ABSENT;
 		}
+	} else {
+		if (pos->unit == MAX_UINT32)
+			pos->unit = 0;
 	}
 
 	return res;
@@ -284,7 +285,7 @@ uint32_t reiser4_node_items(reiser4_node_t *node) {
 			 items, node);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Returns free space of specified node */
 uint16_t reiser4_node_space(reiser4_node_t *node) {
 	aal_assert("umka-455", node != NULL);
@@ -332,10 +333,9 @@ errno_t reiser4_node_shrink(reiser4_node_t *node, pos_t *pos,
 	if ((res = plug_call(node->plug->o.node_ops, shrink,
 			     node, pos, len, count)))
 	{
-		aal_error("Node %llu, pos %u/%u: can't "
-			  "shrink the node on %u bytes.", 
-			  node_blocknr(node), pos->item, 
-			  pos->unit, len);
+		aal_error("Node (%llu), pos (%u/%u): can't shrink "
+			  "the node on (%u) bytes.", node->block->nr,
+			  pos->item, pos->unit, len);
 	}
 
 	return res;
@@ -377,7 +377,6 @@ errno_t reiser4_node_sync(reiser4_node_t *node) {
 
 /* Updates nodeptr item in parent node */
 errno_t reiser4_node_update_ptr(reiser4_node_t *node) {
-	blk_t blk;
 	errno_t res;
 
 	aal_assert("umka-2263", node != NULL);
@@ -385,12 +384,10 @@ errno_t reiser4_node_update_ptr(reiser4_node_t *node) {
 	if (!node->p.node)
 		return 0;
 	
-	blk = node_blocknr(node);
-	
 	if ((res = reiser4_place_fetch(&node->p)))
 		return res;
 	
-	return reiser4_item_update_link(&node->p, blk);
+	return reiser4_item_update_link(&node->p, node->block->nr);
 }
 
 /* Updates node keys in recursive maner (needed for updating ldkeys on the all
@@ -427,9 +424,8 @@ int64_t reiser4_node_modify(reiser4_node_t *node, pos_t *pos,
 	
 	/* Checking if item length is greater then free space in the node. */
 	if (needed > reiser4_node_space(node)) {
-		aal_error("There is no space to insert new "
-			  "item/unit of (%u) size in the node "
-			  "(%llu).", len, node_blocknr(node));
+		aal_error("There is no space to insert new item/unit of (%u) "
+			  "size in the node (%llu).", len, node->block->nr);
 		return -EINVAL;
 	}
 
@@ -440,16 +436,12 @@ int64_t reiser4_node_modify(reiser4_node_t *node, pos_t *pos,
 	return write;
 }
 
-errno_t callback_node_insert(reiser4_node_t *node, pos_t *pos,
-			     trans_hint_t *hint) 
-{
+errno_t cb_node_insert(reiser4_node_t *node, pos_t *pos, trans_hint_t *hint) {
 	return plug_call(node->plug->o.node_ops,
 			 insert, node, pos, hint);
 }
 
-errno_t callback_node_write(reiser4_node_t *node, pos_t *pos,
-			    trans_hint_t *hint) 
-{
+errno_t cb_node_write(reiser4_node_t *node, pos_t *pos, trans_hint_t *hint) {
 	return plug_call(node->plug->o.node_ops,
 			 write, node, pos, hint);
 }
@@ -461,8 +453,7 @@ errno_t reiser4_node_insert(reiser4_node_t *node, pos_t *pos,
 	aal_assert("umka-990", node != NULL);
 	aal_assert("umka-992", hint != NULL);
 	
-	return reiser4_node_modify(node, pos, hint,
-				   callback_node_insert);
+	return reiser4_node_modify(node, pos, hint, cb_node_insert);
 }
 
 int64_t reiser4_node_write(reiser4_node_t *node, pos_t *pos,
@@ -472,8 +463,7 @@ int64_t reiser4_node_write(reiser4_node_t *node, pos_t *pos,
 	aal_assert("umka-2447", hint != NULL);
 	aal_assert("umka-2445", node != NULL);
 
-	return reiser4_node_modify(node, pos, hint,
-				   callback_node_write);
+	return reiser4_node_modify(node, pos, hint, cb_node_write);
 }
 
 int64_t reiser4_node_trunc(reiser4_node_t *node, pos_t *pos,
@@ -505,7 +495,7 @@ errno_t reiser4_node_remove(reiser4_node_t *node, pos_t *pos,
 	{
 		aal_error("Can't remove %llu %s from the node %llu.", 
 			  hint->count, pos->unit == MAX_UINT32 ? 
-			  "items" : "units", node_blocknr(node));
+			  "items" : "units", node->block->nr);
 		return res;
 	}
 
