@@ -839,12 +839,12 @@ static errno_t reiser4_tree_next_key(reiser4_tree_t *tree,
 reiser4_node_t *reiser4_tree_alloc_node(reiser4_tree_t *tree,
 					uint8_t level)
 {
-	blk_t blk;
-	
 	reiser4_node_t *node;
 	uint32_t stamp;
+	errno_t res;
+	blk_t blk;
+	
 	
-	uint64_t free_blocks;
 	reiser4_format_t *format;
     
 	aal_assert("umka-756", tree != NULL);
@@ -854,11 +854,9 @@ reiser4_node_t *reiser4_tree_alloc_node(reiser4_tree_t *tree,
 	format = tree->fs->format;
 
 	/* Setting up of the free blocks in format. */
-	if (!(free_blocks = reiser4_format_get_free(format)))
+	if ((res = reiser4_format_dec_free(format, 1)))
 		return NULL;
 
-	reiser4_format_set_free(format, free_blocks - 1);
-
 	/* Creating new node. */
 	if (!(node = reiser4_node_create(tree, tree->ent.tpset[TPSET_NODE],
 					 blk, level))) 
@@ -879,7 +877,6 @@ reiser4_node_t *reiser4_tree_alloc_node(reiser4_tree_t *tree,
 errno_t reiser4_tree_release_node(reiser4_tree_t *tree,
 				  reiser4_node_t *node)
 {
-	uint64_t free_blocks;
 	reiser4_alloc_t *alloc;
 	reiser4_format_t *format;
 	
@@ -898,8 +895,7 @@ errno_t reiser4_tree_release_node(reiser4_tree_t *tree,
 	}
 
 	/* Setting up of the free blocks in format. */
-	free_blocks = reiser4_format_get_free(format);
-	reiser4_format_set_free(format, free_blocks + 1);
+	reiser4_format_inc_free(format, 1);
 
 	/* Release node itself. */
 	return reiser4_node_close(node);
@@ -988,6 +984,29 @@ static int cb_nodes_comp_func(void *key1, void *key2, void *data) {
 	return 0;
 }
 
+/* Returns the key of the fake root parent */
+errno_t reiser4_tree_root_key(reiser4_tree_t *tree,
+			      reiser4_key_t *key)
+{
+	oid_t locality;
+	oid_t objectid;
+	
+	aal_assert("umka-1949", tree != NULL);
+	aal_assert("umka-1950", key != NULL);
+
+	key->plug = tree->ent.tpset[TPSET_KEY];
+	
+#ifndef ENABLE_MINIMAL
+	locality = reiser4_oid_root_locality(tree->fs->oid);
+	objectid = reiser4_oid_root_objectid(tree->fs->oid);
+#else
+	locality = REISER4_ROOT_LOCALITY;
+	objectid = REISER4_ROOT_OBJECTID;
+#endif
+	return plug_call(key->plug->o.key_ops, build_generic, key,
+			 KEY_STATDATA_TYPE, locality, 0, objectid, 0);
+}
+
 #ifndef ENABLE_MINIMAL
 # define TREE_NODES_TABLE_SIZE (512)
 #else
@@ -1008,7 +1027,6 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
 
 	tree->fs = fs;
 	tree->adjusting = 0;
-	tree->fs->tree = tree;
 
 	/* Initializing hash table for storing loaded formatted nodes in it. */
 	if (!(tree->nodes = aal_hash_table_create(TREE_NODES_TABLE_SIZE,
@@ -1039,7 +1057,7 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
 		goto error_free_data;
 
 	/* Building tree root key. It is used in tree lookup, etc. */
-	if (reiser4_fs_root_key(tree->fs, &tree->key)) {
+	if (reiser4_tree_root_key(tree, &tree->key)) {
 		aal_error("Can't build the tree root key.");
 		goto error_free_data;
 	}
@@ -1057,20 +1075,6 @@ error_free_nodes:
 	return NULL;
 }
 
-#ifndef ENABLE_MINIMAL
-/* Closes specified tree. */
-void reiser4_tree_fini(reiser4_tree_t *tree) {
-	aal_assert("umka-134", tree != NULL);
-
-	/* Allocates everything is needed to allocated and saves dirty nodes to
-	   device. Unloads saved nodes from tree. */
-	reiser4_tree_sync(tree);
-
-	/* Releasing all loaded formatted nodes and tree itself. */
-	reiser4_tree_close(tree);
-}
-#endif
-
 /* Unloads all loaded tree nodes. */
 errno_t reiser4_tree_collapse(reiser4_tree_t *tree) {
         aal_assert("umka-2265", tree != NULL);
@@ -1300,15 +1304,6 @@ static errno_t cb_nodeptr_adjust(reiser4_tree_t *tree, reiser4_place_t *place) {
 #endif
 
 static errno_t cb_node_unload(reiser4_tree_t *tree, reiser4_node_t *node) {
-#ifndef ENABLE_MINIMAL
-	count_t free_blocks;
-
-	/* Updating free space counter in format for the case some blocks were 
-	   allocated. */
-	free_blocks = reiser4_alloc_free(tree->fs->alloc);
-	reiser4_format_set_free(tree->fs->format, free_blocks);
-#endif
-
 	/* If node is locked, that is not a leaf or it is used by someone, it
 	   cannot be released, and thus, it does not make the sense to save it
 	   to device too. */
@@ -2031,7 +2026,7 @@ errno_t reiser4_tree_detach_node(reiser4_tree_t *tree,
 }
 
 /* This function forces tree to grow by one level and sets it up after the
-   growing. This occures when after next balancing root node needs to accept new
+   growing. This occurs when after next balancing root node needs to accept new
    nodeptr item, but has not free space enough.  */
 errno_t reiser4_tree_growup(reiser4_tree_t *tree) {
 	errno_t res;
@@ -2980,7 +2975,8 @@ int64_t reiser4_tree_write(reiser4_tree_t *tree, reiser4_place_t *place,
 }
 
 /* Removes item/unit at @place. */
-errno_t reiser4_tree_remove(reiser4_tree_t *tree, reiser4_place_t *place,
+errno_t reiser4_tree_remove(reiser4_tree_t *tree, 
+			    reiser4_place_t *place,
 			    trans_hint_t *hint)
 {
 	errno_t res;
