@@ -30,18 +30,10 @@ static void repair_cleanup_update(repair_cleanup_t *cleanup) {
 	aal_stream_fini(&stream);
 }
 
-static errno_t cb_free_extent(void *object, uint64_t start, 
-			      uint64_t count, void *data)
-{
+static errno_t cb_free_extent(uint64_t start, uint64_t count, void *data) {
 	repair_cleanup_t *cleanup = (repair_cleanup_t *)data;
 	reiser4_alloc_t *alloc = cleanup->repair->fs->alloc;
-	reiser4_place_t *place = (reiser4_place_t *)object;
 
-	aal_assert("vpf-1420", place != NULL);
-
-	if (reiser4_item_branch(place->plug))
-		return 0;
-	
 	aal_assert("vpf-1418", reiser4_alloc_occupied(alloc, start, count));
 
 	reiser4_alloc_release(alloc, start, count);
