@@ -12,6 +12,7 @@ errno_t repair_fs_open(repair_data_t *repair,
 		       aal_device_t *hdevice,
 		       aal_device_t *jdevice)
 {
+	reiser4_fs_t *fs;
 	errno_t res = 0;
 
 	aal_assert("vpf-851",  repair != NULL);
@@ -19,89 +20,105 @@ errno_t repair_fs_open(repair_data_t *repair,
 	aal_assert("vpf-1556", jdevice != NULL);
  
 	/* Allocating memory and initializing fields */
-	if (!(repair->fs = aal_calloc(sizeof(*repair->fs), 0)))
+	if (!(fs = repair->fs = aal_calloc(sizeof(*repair->fs), 0)))
 		return -ENOMEM;
 
-	repair->fs->device = hdevice;
+	fs->device = hdevice;
 	
-	res |= repair_master_open(repair->fs, repair->mode);
+	/* Try to open master & format -- evth what gets backed up. */
+	fs->master = reiser4_master_open(fs->device);
+	if (fs->master) {
+		fs->format = reiser4_format_open(fs);
+	}
+	
+	/* Try to open the reiser4 backup. */
+	if (!(fs->backup = repair_backup_open(fs, repair->mode))) {
+		if (repair->mode != RM_BUILD) {
+			aal_fatal("Failed to open the reiser4 backup.");
+			res = RE_FATAL;
+			goto error_fs_close;
+		}
+	}
+	
+	res |= repair_master_check_struct(fs, repair->mode, repair->flags);
 	
 	if (repair_error_fatal(res)) {
 		aal_fatal("Failed to open the master super block.");
-		goto error_fs_free;
+		goto error_fs_close;
 	}
 	
-	res |= repair_format_open(repair->fs, repair->mode);
+	res |= repair_format_check_struct(fs, repair->mode, repair->flags);
 	
 	if (repair_error_fatal(res)) {
 		aal_fatal("Failed to open the format.");
-		goto error_master_close;
+		goto error_fs_close;
 	}
 
 	/* FIXME-VITALY: if status has io flag set and there is no bad
 	   block file given to fsck -- do not continue -- when bad block 
 	   support will be written. */
-	res |= repair_status_open(repair->fs, repair->mode);
+	res |= repair_status_open(fs, repair->mode);
 	
 	if (repair_error_fatal(res)) {
 		aal_fatal("Failed to open the status block.");
-		goto error_format_close;
+		goto error_fs_close;
 	}
 	
-	res |= repair_alloc_open(repair->fs, repair->mode);
+	res |= repair_alloc_open(fs, repair->mode);
 
 	if (repair_error_fatal(res)) {
 		aal_fatal("Failed to open the block allocator.");
 		goto error_status_close;
 	}
 	
-	if (!(repair->fs->oid = reiser4_oid_open(repair->fs))) {	
+	if (!(fs->oid = reiser4_oid_open(fs))) {
 		aal_fatal("Failed to open an object id allocator.");
 		res = -EINVAL;
 		goto error_alloc_close;
 	}
 
-	res |= repair_journal_open(repair->fs, jdevice, repair->mode);
+	res |= repair_journal_open(fs, jdevice, repair->mode, repair->flags);
 	
 	if (repair_error_fatal(res)) {
 		aal_fatal("Failed to open the journal.");
 		goto error_oid_close;
 	}
 	
-	if (!(repair->fs->tree = reiser4_tree_init(repair->fs))) {
+	if (!(fs->tree = reiser4_tree_init(fs))) {
 		aal_fatal("Failed to init the fs-global plugin set.");
 		res = -ENOMEM;
 		goto error_journal_close;
 	}
-	
+
 	repair_error_count(repair, res);
 	return 0;
 
-error_journal_close:
-	reiser4_journal_close(repair->fs->journal);
-	repair->fs->journal = NULL;
+ error_journal_close:
+	reiser4_journal_close(fs->journal);
+	fs->journal = NULL;
 
  error_oid_close:
-	reiser4_oid_close(repair->fs->oid);
+	reiser4_oid_close(fs->oid);
 	
  error_alloc_close:
-	reiser4_alloc_close(repair->fs->alloc);
-	repair->fs->alloc = NULL;
+	reiser4_alloc_close(fs->alloc);
+	fs->alloc = NULL;
 
  error_status_close:
-	reiser4_status_close(repair->fs->status);
-	repair->fs->status = NULL;
+	reiser4_status_close(fs->status);
+	fs->status = NULL;
 
- error_format_close:
-	reiser4_format_close(repair->fs->format);
-	repair->fs->format = NULL;
+ error_fs_close:
+	if (fs->backup)
+		reiser4_backup_close(fs->backup);
+	
+	if (fs->format)
+		reiser4_format_close(fs->format);
+	
+	if (fs->master)
+		reiser4_master_close(fs->master);
 
- error_master_close:
-	reiser4_master_close(repair->fs->master);
-	repair->fs->master = NULL;
-
- error_fs_free:
-	aal_free(repair->fs);
+	aal_free(fs);
 	repair->fs = NULL;
 
 	repair_error_count(repair, res);
@@ -129,16 +146,6 @@ errno_t repair_fs_replay(reiser4_fs_t *fs) {
 	return 0;
 }
 
-/* Close the journal and the filesystem. */
-void repair_fs_close(reiser4_fs_t *fs) {
-	aal_assert("vpf-909", fs != NULL);
-	aal_assert("vpf-910", fs->journal != NULL);
-	
-	reiser4_journal_close(fs->journal);
-	fs->journal = NULL;
-	reiser4_fs_close(fs);    
-}
-
 /* Pack passed @fs to @stream. */
 errno_t repair_fs_pack(reiser4_fs_t *fs, 
 		       aux_bitmap_t *bitmap, 
@@ -177,6 +184,11 @@ errno_t repair_fs_pack(reiser4_fs_t *fs,
 	if ((res = repair_backup_pack(fs, stream)))
 		return res;
 	
+	aal_stream_write(stream, JOURNAL_PACK_SIGN, 4);
+	
+	if ((res = repair_journal_pack(fs->journal, stream)))
+		return res;
+	
 	len = reiser4_format_get_len(fs->format);
 
 	/* Loop though the all data blocks, check if they belong to tree and if
@@ -198,22 +210,23 @@ errno_t repair_fs_pack(reiser4_fs_t *fs,
 		if (!(node = reiser4_node_open(fs->tree, blk)))
 			continue;
 
-		if ((res = repair_node_check_struct(node, NULL, 
-						    RM_CHECK, NULL)) < 0)
-			return res;
+		res = repair_node_check_struct(node, NULL, RM_CHECK, NULL);
 
-		if (res) {
+		if (res > 0) {
 			aal_stream_write(stream, BLOCK_PACK_SIGN, 4);
 		
 			/* Packing @node to @stream. */
-			if ((res = repair_node_pack(node, stream, PACK_OFF)))
+			if ((res = repair_fs_block_pack(node->block, stream)))
 				return res;
-		} else {
+		} else if (res == 0) {
 			aal_stream_write(stream, NODE_PACK_SIGN, 4);
 		
 			/* Packing @node to @stream. */
-			if ((res = repair_node_pack(node, stream, PACK_FULL)))
+			if ((res = repair_node_pack(node, stream)))
 				return res;
+		} else {
+			reiser4_node_close(node);
+			return res;
 		}
 
 		/* Close node. */
@@ -223,9 +236,7 @@ errno_t repair_fs_pack(reiser4_fs_t *fs,
 	return 0;
 }
 
-static errno_t cb_mark_used(void *object, uint64_t start,
-			    uint64_t count, void *data) 
-{
+static errno_t cb_mark_used(uint64_t start, uint64_t count, void *data) {
 	aux_bitmap_t *bitmap = (aux_bitmap_t *)data;
 
 	aux_bitmap_mark_region(bitmap, start, count);
@@ -244,6 +255,7 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 	char sign[5] = {0};
 	
 	aal_block_t *block;
+	reiser4_node_t *node;
 	
 	aal_assert("umka-2633", device != NULL);
 	aal_assert("umka-2648", stream != NULL);
@@ -295,11 +307,11 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 	if (aal_block_write(block)) {
 		aal_error("Can't write the very last block (%llu) "
 			  "on the fs: %s", bn, device->error);
-		aal_free(block);
+		aal_block_free(block);
 		goto error_free_format;
 	}
 
-	aal_free(block);
+	aal_block_free(block);
 	
 	if (!(fs->oid = reiser4_oid_open(fs)))
 		goto error_free_format;
@@ -341,7 +353,7 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 	}
 
 	if (aal_strncmp(sign, BACKUP_PACK_SIGN, 4)) {
-		aal_error("Invalid backup blocks sign %s is "
+		aal_error("Invalid backup sign %s is "
 			  "detected in stream.", sign);
 		goto error_free_status;
 	}
@@ -349,6 +361,20 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 	if (repair_backup_unpack(fs, stream))
 		goto error_free_status;
 
+	if (aal_stream_read(stream, &sign, 4) != 4) {
+		aal_error("Can't unpack journal blocks. Stream is over?");
+		goto error_free_backup;
+	}
+	
+	if (aal_strncmp(sign, JOURNAL_PACK_SIGN, 4)) {
+		aal_error("Invalid journal sign %s is "
+			  "detected in stream.", sign);
+		goto error_free_backup;
+	}
+
+	if (repair_journal_unpack(fs, stream))
+		goto error_free_backup;
+	
 	/* If @bitmap is given, save there unpacked blocks of the fs. */
 	if (bitmap) {
 		uint64_t len = reiser4_format_get_len(fs->format);
@@ -359,52 +385,70 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 		if (reiser4_fs_layout(fs, cb_mark_used, bitmap)) {
 			aal_error("Can't to mark all frozen fs "
 				  "blocks as used in the bitmap.");
-			goto error_free_status;
+			goto error_free_journal;
 		}
 	}
 		
 	while (1) {
-		reiser4_node_t *node;
+		int pack;
 		
 		if (aal_stream_read(stream, &sign, 4) != 4) {
 			if (aal_stream_eof(stream))
 				break;
 			else
-				goto error_free_status;
+				goto error_free_journal;
 		}
 
+		node = NULL;
+		block = NULL;
+		
 		if (!aal_strncmp(sign, NODE_PACK_SIGN, 4)) {
-			node = repair_node_unpack(fs->tree, stream, PACK_FULL);
+			node = repair_node_unpack(fs->tree, stream);
+			pack = 1;
 		} else if (!aal_strncmp(sign, BLOCK_PACK_SIGN, 4)) {
-			node = repair_node_unpack(fs->tree, stream, PACK_OFF);
+			block = repair_fs_block_unpack(fs, stream);
+			pack = 0;
 		} else {
 			aal_error("Invalid object %s is detected in stream. "
 				  "Node is expected.", sign);
-			goto error_free_status;
+			goto error_free_journal;
 		}
 
-		if (!node) goto error_free_status;
-
-		if (reiser4_node_sync(node)) {
-			reiser4_node_close(node);
-			goto error_free_status;
+		if ((pack && !node) || (!pack && !block)) {
+			goto error;
 		}
 
+		if (pack && reiser4_node_sync(node))
+			goto error;
+		else if (!pack && aal_block_write(block))
+			goto error_free_journal;
+
 		if (bitmap) {
 			aux_bitmap_mark(bitmap, node->block->nr);
 		}
 		
-		reiser4_node_close(node);
+		if (pack) {
+			reiser4_node_close(node);
+		} else {
+			aal_block_free(block);
+		}
 	}
 
 	return fs;
 
+ error:
+	if (block) aal_block_free(block);
+	if (node) reiser4_node_close(node);
+ error_free_journal:
+	reiser4_journal_close(fs->journal);
+ error_free_backup:
+	reiser4_backup_close(fs->backup);
  error_free_status:
 	reiser4_status_close(fs->status);
  error_free_alloc:
 	reiser4_alloc_close(fs->alloc);
  error_free_tree:
-	reiser4_tree_fini(fs->tree);
+	reiser4_tree_close(fs->tree);
  error_free_oid:
 	reiser4_oid_close(fs->oid);
  error_free_format:
@@ -430,3 +474,68 @@ errno_t repair_fs_lost_key(reiser4_fs_t *fs, reiser4_key_t *key) {
 	return reiser4_key_build_generic(key, KEY_STATDATA_TYPE,
 					 locality, 0, objectid, 0);
 }
+
+/* Fs is not opened yet. A block is read and it could be a backup one.
+   This method figure out if it is. */
+errno_t repair_fs_check_backup(aal_device_t *device, backup_hint_t *hint) {
+	errno_t res;
+
+	aal_assert("vpf-1730", hint != NULL);
+	aal_assert("vpf-1730", hint->block.data != NULL);
+	
+	/* Check the backup version. */
+	if (((char *)hint->block.data)[0] != 0)
+		return RE_FATAL;
+
+	/* Master backup starts on 1st byte. Note: Every backuper must set 
+	   hint->el[next index] correctly. */
+	hint->el[BK_MASTER] = hint->block.data + 1;
+
+	/* Check the master backup structure. */
+	if ((res = repair_master_check_backup(hint)))
+		return res;
+
+	/* Backup the format backup structure. */
+	return repair_format_check_backup(device, hint);
+}
+
+errno_t repair_fs_block_pack(aal_block_t *block, aal_stream_t *stream) {
+	aal_assert("vpf-1749", block != NULL);
+	aal_assert("vpf-1750", stream != NULL);
+
+	aal_stream_write(stream, &block->nr, sizeof(block->nr));
+	aal_stream_write(stream, block->data, block->size);
+
+	return 0;
+}
+
+aal_block_t *repair_fs_block_unpack(reiser4_fs_t *fs, aal_stream_t *stream) {
+	aal_block_t *block;
+	count_t size;
+	count_t read;
+	blk_t blk;
+	
+	
+	aal_assert("vpf-1751", fs != NULL);
+	aal_assert("vpf-1752", stream != NULL);
+
+	if (aal_stream_read(stream, &blk, sizeof(blk)) != sizeof(blk))
+		return NULL;
+	
+	size = reiser4_master_get_blksize(fs->master);
+
+	/* Allocate new node of @size at @nr. */
+	if (!(block = aal_block_alloc(fs->device, size, blk)))
+		return NULL;
+
+	read = aal_stream_read(stream, block->data, block->size);
+	
+	if (read != block->size)
+		goto error;
+
+	return block;
+	
+ error:
+	aal_block_free(block);
+	return NULL;
+}
