@@ -29,6 +29,38 @@ typedef enum repair_mark {
 	RM_CLEAR = 2
 } repair_mark_t;
 
+void cb_gauge_tree_percent(aal_gauge_t *gauge) {
+	uint32_t pos[REISER4_TREE_MAX_HEIGHT][2];
+	uint64_t amount, passed;
+	reiser4_place_t *place;
+	uint8_t i;
+	
+	if (!gauge || !gauge->data)
+		return;
+	
+	place = &((reiser4_node_t *)gauge->data)->p;
+	
+	i = 0;
+	while (place->node) {
+		pos[i][0] = place->pos.item;
+		pos[i][1] = reiser4_node_items(place->node);
+		place = &place->node->p;
+		i++;
+	}
+	
+	passed = amount = 0;
+	
+	while (i) {
+		i--;
+		amount = amount ? amount * pos[i][1] : pos[i][1];
+		
+		if (passed) passed *= pos[i][1];
+		passed += pos[i][0];
+	}
+
+	aal_gauge_set_value(gauge, amount ? passed * 100 / amount : 0);
+}
+
 static void repair_filter_node_handle(repair_filter_t *fd, blk_t blk, 
 				      uint8_t level, repair_mark_t mark)
 {
@@ -180,8 +212,8 @@ static reiser4_node_t *repair_filter_node_open(reiser4_tree_t *tree,
 		return INVAL_PTR;
 	
 	if (blk >= fd->bm_used->total) {
-		aal_error("Node (%llu), item (%u), unit (%u): Points to the "
-			  "invalid block (%llu).%s", node_blocknr(place->node),
+		fsck_mess("Node (%llu), item (%u), unit (%u): Points to the "
+			  "invalid block (%llu).%s", place_blknr(place),
 			  place->pos.item, place->pos.unit, blk, 
 			  fd->repair->mode == RM_BUILD ? " Removed." :
 			  " The whole subtree is skipped.");
@@ -190,9 +222,9 @@ static reiser4_node_t *repair_filter_node_open(reiser4_tree_t *tree,
 	
 	if (aux_bitmap_test_region(fd->bm_used, blk, 1, 1)) {
 		/* Bad pointer detected. Remove if possible. */
-		aal_error("Node (%llu), item (%u), unit (%u): Points to the "
+		fsck_mess("Node (%llu), item (%u), unit (%u): Points to the "
 			  "block (%llu) which is in the tree already.%s", 
-			  node_blocknr(place->node), place->pos.item, 
+			  place_blknr(place), place->pos.item, 
 			  place->pos.unit, blk, fd->repair->mode == RM_BUILD ?
 			  " Removed." : " The whole subtree is skipped.");
 		error = 1;
@@ -201,27 +233,17 @@ static reiser4_node_t *repair_filter_node_open(reiser4_tree_t *tree,
 	if (error) goto error;
 	
 	if (!(node = repair_tree_load_node(fd->repair->fs->tree, place->node, 
-					   blk, *fd->check_node))) 
+					   blk, fd->mkidok ? fd->mkid : 0)))
 	{
-		aal_error("Node (%llu): failed to open the node pointed by the "
+		fsck_mess("Node (%llu): failed to open the node pointed by the "
 			  "node (%llu), item (%u), unit (%u) on the level (%u)."
 			  " The whole subtree is skipped.", blk, 
-			  node_blocknr(place->node), place->pos.item, 
+			  place_blknr(place), place->pos.item, 
 			  place->pos.unit, reiser4_node_get_level(place->node));
 		goto error;
 	}
 	
-	if (fd->progress_handler && fd->level != LEAF_LEVEL) {
-		fd->progress->state = PROGRESS_UPDATE;
-		fd->progress->u.tree.i_total = reiser4_node_items(place->node);
-		fd->progress->u.tree.u_total = reiser4_item_units(place);
-		fd->progress->u.tree.item = place->pos.item;
-		fd->progress->u.tree.unit = place->pos.unit;
-		fd->progress_handler(fd->progress);
-	}
-	
 	repair_filter_read_node(fd, blk, reiser4_node_get_level(node));
-
 	return node;
 	
  error:
@@ -229,15 +251,22 @@ static reiser4_node_t *repair_filter_node_open(reiser4_tree_t *tree,
 	return NULL;
 }
 
+static errno_t cb_count_sd(reiser4_place_t *place, void *data) {
+	repair_filter_t *fd = (repair_filter_t *)data;
+
+	if (place->plug->id.group == STAT_ITEM)
+		fd->stat.tmp++;
+
+	return 0;
+}
+
 /* Before callback for traverse. It checks node level, node consistency, and 
    delimiting keys. If any check reveals a problem with the data consistency 
    it sets RE_FATAL flag. */
-static errno_t repair_filter_node_check(reiser4_tree_t *tree,
-					reiser4_node_t *node,
-					void *data)
-{
+static errno_t repair_filter_node_check(reiser4_node_t *node, void *data) {
 	repair_filter_t *fd = (repair_filter_t *)data;
 	errno_t res = 0;
+	uint32_t items;
 	uint16_t level;
     
 	aal_assert("vpf-252", data  != NULL);
@@ -252,40 +281,35 @@ static errno_t repair_filter_node_check(reiser4_tree_t *tree,
 		fd->level = level;
 	else
 		fd->level--;
-	
-	if (fd->progress_handler && fd->level != LEAF_LEVEL) {
-		fd->progress->state = PROGRESS_START;
-		fd->progress->u.tree.i_total = reiser4_node_items(node);
-		fd->progress->u.tree.u_total = 0;
-		fd->progress->u.tree.item = 0;
-		fd->progress->u.tree.unit = 0;
-		fd->progress_handler(fd->progress);
-	}
 
 	/* Check the level. */
 	if (fd->level != level) {
-		aal_error("Level (%u) of the node (%llu) doesn't match the "
-			  "expected one (%u). %s", level, node_blocknr(node), 
+		fsck_mess("Level (%u) of the node (%llu) doesn't match the "
+			  "expected one (%u). %s", level, node->block->nr, 
 			  fd->level, fd->repair->mode == RM_BUILD ? 
 			  "Removed." : "The whole subtree is skipped.");
 		
 		/* Should not be check for now as it may lie in unused space.
 		   It is just a wrong pointer. Skip it. */
 		
-		repair_filter_bad_level(fd, node_blocknr(node), level);
+		repair_filter_bad_level(fd, node->block->nr, level);
 		goto error;
 	} 
 	
-	if ((res = repair_node_check_struct(node, fd->repair->mode)) < 0)
+	if ((res = repair_node_check_struct(node, cb_count_sd, 
+					    fd->repair->mode, fd)) < 0)
 		return res;
 	
 	if (!(res & RE_FATAL)) {
+		(*fd->stat.files) += fd->stat.tmp;
+		fd->stat.tmp = 0;
+
 		res |= repair_node_check_level(node, fd->repair->mode);
 		if (res < 0) return res;
 	}
 	
-	if (reiser4_node_items(node) == 0) {
-		repair_filter_empty_node(fd, node_blocknr(node), level);
+	if ((items = reiser4_node_items(node)) == 0) {
+		repair_filter_empty_node(fd, node->block->nr, level);
 		reiser4_node_mkclean(node);
 		goto error;
 	}
@@ -293,7 +317,7 @@ static errno_t repair_filter_node_check(reiser4_tree_t *tree,
 	repair_error_count(fd->repair, res);
 	
 	if (res & RE_FATAL) {
-		repair_filter_bad_node(fd, node_blocknr(node), level);
+		repair_filter_bad_node(fd, node->block->nr, level);
 		goto error;
 	} else if (res == 0) {
 		if (reiser4_node_isdirty(node))
@@ -301,11 +325,14 @@ static errno_t repair_filter_node_check(reiser4_tree_t *tree,
 	}
 	
 	/* There are no fatal errors, check delimiting keys. */
-	if ((res = repair_tree_dknode_check(tree, node, fd->repair->mode)) < 0)
+	if ((res = repair_tree_dknode_check((reiser4_tree_t *)node->tree,
+					    node, fd->repair->mode)) < 0)
+	{
 		return res;
+	}
 	
 	if (res) {
-		repair_filter_bad_dk(fd, node_blocknr(node), level);
+		repair_filter_bad_dk(fd, node->block->nr, level);
 		goto error;
 	}
 	
@@ -313,15 +340,12 @@ static errno_t repair_filter_node_check(reiser4_tree_t *tree,
 	if (fd->repair->mode == RM_BUILD)
 		repair_node_clear_flags(node);
 	
+	aal_gauge_set_data(fd->gauge, node);
+	aal_gauge_touch(fd->gauge);
+	
 	return 0;
  error:
-	if (fd->level != LEAF_LEVEL) {
-		fd->progress->state = PROGRESS_END;
-		fd->progress_handler(fd->progress);
-	}
-	
 	fd->level++;
-	
 	return RE_FATAL;
 }
 
@@ -329,21 +353,25 @@ static errno_t repair_filter_node_check(reiser4_tree_t *tree,
    callback and do some essential stuff after traversing through the child -
    level, if RE_PTR flag is set - deletes the child pointer and mark
    the pointed block as unused in bm_used bitmap. */
-static errno_t repair_filter_update_traverse(reiser4_tree_t *tree, 
-					     reiser4_place_t *place, 
+static errno_t repair_filter_update_traverse(reiser4_place_t *place, 
 					     void *data) 
 {
 	repair_filter_t *fd = (repair_filter_t *)data;
 	reiser4_node_t *node;
+	reiser4_tree_t *tree;
 	errno_t res;
 	blk_t blk;
     
 	aal_assert("vpf-257", fd != NULL);
 	aal_assert("vpf-434", place != NULL);
+	aal_assert("vpf-434", place->node != NULL);
+	aal_assert("vpf-434", place->node->tree != NULL);
+
+	tree = (reiser4_tree_t *)place->node->tree;
 
 	if ((blk = reiser4_item_down_link(place)) == INVAL_BLK) {
-		aal_fatal("Node (%llu), item (%u), unit(%u): Failed to fetch "
-			  "the node pointer.", node_blocknr(place->node),
+		aal_error("Node (%llu), item (%u), unit(%u): Failed to "
+			  "fetch the node pointer.", place_blknr(place),
 			  place->pos.item, place->pos.unit);
 		return -EIO;
 	}
@@ -352,20 +380,20 @@ static errno_t repair_filter_update_traverse(reiser4_tree_t *tree,
 		return 0;
 
 	if ((fd->flags & RE_FATAL) || (fd->flags & RE_EMPTY)) {
-		aal_error("Node (%llu): the node is %s. Pointed from "
+		fsck_mess("Node (%llu): the node is %s. Pointed from "
 			  "the node (%llu), item (%u), unit (%u). %s",
 			  blk, fd->flags & RE_EMPTY ? "empty" :
 			  fd->repair->mode == RM_BUILD ? "unrecoverable" : 
-			  "broken", node_blocknr(place->node), place->pos.item,
+			  "broken", place_blknr(place), place->pos.item,
 			  place->pos.unit, fd->repair->mode == RM_BUILD ? 
 			  "Removed." : "The whole subtree is skipped.");
 	} else if (fd->flags & RE_DKEYS) {
-		aal_error("Node (%llu), item (%u), unit (%u): Points to "
+		fsck_mess("Node (%llu), item (%u), unit (%u): Points to "
 			  "the node [%llu] with wrong delimiting keys. %s",
-			  node_blocknr(place->node), place->pos.item, 
+			  place_blknr(place), place->pos.item, 
 			  place->pos.unit, blk, fd->repair->mode == RM_BUILD ?
 			  "Removed, content will be inserted later item-by-"
-			  "-item." : "The whole subtree is skipped.");
+			  "item." : "The whole subtree is skipped.");
 	}
 	
 	/* In the case of an error the node should be closed as it should 
@@ -409,45 +437,23 @@ static errno_t repair_filter_update_traverse(reiser4_tree_t *tree,
 /* After callback for traverse. Does needed stuff after traversing through all 
    children - if no child left, set RE_PTR flag to force deletion of the 
    pointer to this block in update_traverse callback. */
-static errno_t repair_filter_after_traverse(reiser4_tree_t *tree, 
-					    reiser4_node_t *node, 
-					    void *data) 
-{
+static errno_t repair_filter_after_traverse(reiser4_node_t *node, void *data) {
 	repair_filter_t *fd = (repair_filter_t *)data;
 	
 	aal_assert("vpf-393", node != NULL);
 	aal_assert("vpf-256", fd != NULL);    
 	
 	if (reiser4_node_items(node) == 0) {
-		repair_filter_empty_node(fd, node_blocknr(node), 
+		repair_filter_empty_node(fd, node->block->nr, 
 					 reiser4_node_get_level(node));
 		reiser4_node_mkclean(node);
+	} else if (!fd->mkidok && fd->mkid != reiser4_node_get_mstamp(node)) {
+		/* If mkfsid is a new one, set it to the node. */
+		reiser4_node_set_mstamp(node, fd->mkid);
 	}
-	
-	if (fd->progress_handler && fd->level != LEAF_LEVEL) {
-		fd->progress->state = PROGRESS_END;
-		fd->progress_handler(fd->progress);
-	}
-	
-	fd->level++;
-	
-	return 0;
-}
 
-/* Setup data (common and specific) before traverse through the tree. */
-static void repair_filter_setup(repair_filter_t *fd) {
-	aal_memset(fd->progress, 0, sizeof(*fd->progress));
-	
-	if (!fd->progress_handler)
-		return;
-	
-	fd->progress->type = GAUGE_TREE;
-	fd->progress->text = "***** Tree Traverse Pass: scanning the reiser4 "
-		"internal tree.";
-	fd->progress->state = PROGRESS_STAT;
-	time(&fd->stat.time);
-	fd->progress_handler(fd->progress);
-	fd->progress->text = NULL;
+	fd->level++;
+	return 0;
 }
 
 /* Does some update stuff after traverse through the internal tree - 
@@ -467,8 +473,11 @@ static void repair_filter_update(repair_filter_t *fd) {
 	format = fd->repair->fs->format;
 	
 	if (fd->flags) {
+		reiser4_tree_t *tree;
+		reiser4_node_t *node;
+		
 		if (!(fd->flags & RE_PTR)) {
-			aal_error("Root node (%llu): the node is %s. %s",
+			fsck_mess("Root node (%llu): the node is %s. %s",
 				  reiser4_format_get_root(format), 
 				  fd->flags & RE_EMPTY ? "empty" :
 				  fd->repair->mode == RM_BUILD ? 
@@ -480,17 +489,20 @@ static void repair_filter_update(repair_filter_t *fd) {
 			aal_warn("Reiser4 storage tree does not exist. "
 				 "Filter pass skipped.");
 		}
+	
+		tree = fd->repair->fs->tree;
+		node = fd->repair->fs->tree->root;
+		
+		if (node) {
+			reiser4_tree_disconnect_node(tree, node);
+
+			/* If there is another pointer to this node, 
+			   changes should be saved. */
+			reiser4_node_fini(node);
+			tree->root = NULL;
+		}
 
 		if (fd->repair->mode == RM_BUILD) {
-			reiser4_node_t *root;
-			
-			root = fd->repair->fs->tree->root;
-			
-			if (root) {			
-				reiser4_node_fini(root);			
-				fd->repair->fs->tree->root = NULL;
-			}
-			
 			reiser4_format_set_root(format, INVAL_BLK);
 			fd->repair->fatal--;
 		}
@@ -498,23 +510,24 @@ static void repair_filter_update(repair_filter_t *fd) {
 
 	/* Check the tree height. */
 	height = reiser4_format_get_height(fd->repair->fs->format);
-	fd->level--;
-	if (height != fd->level) {
-		aal_mess("The tree height %u found in the format is wrong. "
-			 "%s %u.", height, fd->repair->mode == RM_CHECK ? 
-			 "Should be" : "Fixed to", fd->level);
+	
+	if (fd->level) {
+		fd->level--;
 
-		if (fd->repair->mode == RM_CHECK) {
-			fd->repair->fixable++;
-		} else {
-			reiser4_format_set_height(fd->repair->fs->format, 
-						  fd->level);
+		if (height != fd->level) {
+			fsck_mess("The tree height %u found in the format is "
+				  "wrong. %s %u.", height, fd->repair->mode == 
+				  RM_CHECK ? "Should be":"Fixed to", fd->level);
+
+			if (fd->repair->mode == RM_CHECK) {
+				fd->repair->fixable++;
+			} else {
+				reiser4_format_set_height(fd->repair->fs->format,
+							  fd->level);
+			}
 		}
 	}
 	
-	if (!fd->progress_handler)
-		return;
-
 	aal_stream_init(&stream, NULL, &memory_stream);
 	
 	aal_stream_format(&stream, "\tRead nodes %llu\n", 
@@ -572,11 +585,7 @@ static void repair_filter_update(repair_filter_t *fd) {
 	time_str = ctime(&fd->stat.time);
 	time_str[aal_strlen(time_str) - 1] = '\0';
 	aal_stream_format(&stream, time_str);
-
-	fd->progress->state = PROGRESS_STAT;
-	fd->progress->text = (char *)stream.entity;
-	fd->progress_handler(fd->progress);
-
+	aal_mess(stream.entity);
 	aal_stream_fini(&stream);
 }
 
@@ -603,23 +612,19 @@ static errno_t repair_filter_traverse(repair_filter_t *fd) {
 	}
 	
 	/* try to open the root node. */
-	if (!(tree->root = repair_tree_load_node(fd->repair->fs->tree, 
-						 NULL, root, 0)))
-	{
-		aal_error("Node (%llu): failed to open the root node. "
+	tree->root = repair_tree_load_node(fd->repair->fs->tree, NULL,
+					   root, fd->mkidok ? fd->mkid: 0);
+	
+	if (!tree->root) {
+		fsck_mess("Node (%llu): failed to open the root node. "
 			  "The whole filter pass is skipped.", root);
 		
 		goto error;
 	}
 	
 	repair_filter_read_node(fd, root, reiser4_node_get_level(tree->root));
-	
-	/* If SB's mkfs id exists and matches the root node's one, 
-	   check the mkfs id of all nodes. */
-	*fd->check_node = (reiser4_format_get_stamp(format) && 
-			   (reiser4_format_get_stamp(format) ==
-			    reiser4_node_get_mstamp(tree->root)));
-	
+	aal_gauge_touch(fd->gauge);
+
 	/* Cut the corrupted, unrecoverable parts of the tree off. */
 	res = reiser4_tree_trav_node(tree, tree->root,
 				     repair_filter_node_open,
@@ -627,6 +632,8 @@ static errno_t repair_filter_traverse(repair_filter_t *fd) {
 				     repair_filter_update_traverse,  
 				     repair_filter_after_traverse, fd);
 
+	aal_gauge_done(fd->gauge);
+	
 	return res < 0 ? res : 0;
  error:
 	repair_filter_bad_ptr(fd);
@@ -638,7 +645,6 @@ static errno_t repair_filter_traverse(repair_filter_t *fd) {
    corrupted parts off, and fixing what can be fixed. Account all kind of 
    nodes in corresponding bitmaps. */
 errno_t repair_filter(repair_filter_t *fd) {
-	repair_progress_t progress;
 	errno_t res = 0;
 
 	aal_assert("vpf-536", fd != NULL);
@@ -647,12 +653,16 @@ errno_t repair_filter(repair_filter_t *fd) {
 	aal_assert("vpf-816", fd->repair->fs->tree != NULL);
 	aal_assert("vpf-815", fd->bm_used != NULL);
 
-	fd->progress = &progress;
-	repair_filter_setup(fd);
+	aal_mess("CHECKING STORAGE TREE");
+	fd->gauge = aal_gauge_create(aux_gauge_handlers[GT_PROGRESS], 
+				     cb_gauge_tree_percent, NULL, 500, NULL);
+	time(&fd->stat.time);
 	
 	res = repair_filter_traverse(fd);
 	
+	aal_gauge_free(fd->gauge);
 	repair_filter_update(fd);
+	
 	if (fd->repair->mode != RM_CHECK)
 		reiser4_fs_sync(fd->repair->fs);
 	
