@@ -23,7 +23,8 @@ typedef struct repair_control {
 	
 	aux_bitmap_t *bm_alloc;
 
-	bool_t check_node;
+	bool_t mkidok;
+	uint32_t mkid;
 	uint64_t oid, files;
 } repair_control_t;
 
@@ -87,17 +88,15 @@ static errno_t repair_bitmap_compare(aux_bitmap_t *bm1, aux_bitmap_t *bm2,
 }
 
 /* Callback for the format_ops.layout method - mark all blocks in the bitmap. */
-static errno_t callback_format_mark(void *format, blk_t start,
-				    count_t width, void *data)
-{
+static errno_t cb_format_mark(blk_t start, count_t width, void *data) {
 	aux_bitmap_t *format_layout = (aux_bitmap_t *)data;
 	aux_bitmap_mark_region(format_layout, start, width);
 	return 0;
 }
 
 
-static errno_t callback_alloc(reiser4_alloc_t *alloc, uint64_t start, 
-			      uint64_t count, void *data) 
+static errno_t cb_alloc(reiser4_alloc_t *alloc, uint64_t start, 
+			uint64_t count, void *data) 
 {
 	repair_control_t *control = (repair_control_t *)data;
 	
@@ -108,8 +107,8 @@ static errno_t callback_alloc(reiser4_alloc_t *alloc, uint64_t start,
 	return 0;
 }
 
-static errno_t callback_release(reiser4_alloc_t *alloc, uint64_t start, 
-				uint64_t count, void *data) 
+static errno_t cb_release(reiser4_alloc_t *alloc, uint64_t start, 
+			  uint64_t count, void *data) 
 {
 	repair_control_t *control = (repair_control_t *)data;
 	
@@ -123,6 +122,7 @@ static errno_t callback_release(reiser4_alloc_t *alloc, uint64_t start,
 static errno_t repair_filter_prepare(repair_control_t *control, 
 				     repair_filter_t *filter) 
 {
+	reiser4_fs_t *fs;
 	uint64_t fs_len;
 	
 	aal_assert("vpf-592", filter != NULL);
@@ -131,38 +131,42 @@ static errno_t repair_filter_prepare(repair_control_t *control,
 	aal_assert("vpf-839", control->repair->fs != NULL);
 	
 	aal_memset(filter, 0, sizeof(*filter));    
+	fs = control->repair->fs;
+
+	/* If backup has not been opened, do not check the mkfs id. */
+	control->mkid = reiser4_format_get_stamp(fs->format);
+	control->mkidok = fs->backup || 
+		control->repair->mode != RM_BUILD ? 1 : 0;
+	
 	filter->repair = control->repair;
-	filter->check_node = &control->check_node;
+	filter->stat.files = &control->files;
+	filter->mkidok = control->mkidok;
+	filter->mkid = control->mkid;
 	
-	filter->progress_handler = control->repair->progress_handler;
-	
-	fs_len = reiser4_format_get_len(control->repair->fs->format);
+	fs_len = reiser4_format_get_len(fs->format);
 	
 	/* Allocate a bitmap of blocks belong to the format area - skipped, 
 	   super block, journal, bitmaps. */
-	if (!(control->bm_used = filter->bm_used = aux_bitmap_create(fs_len))) {
-		aal_error("Failed to allocate a bitmap of format "
-			  "layout.");
+	if (!(control->bm_used = filter->bm_used = aux_bitmap_create(fs_len))) 
+	{
+		aal_error("Failed to allocate a bitmap of format layout.");
 		return -EINVAL;
 	}
 	
 	/* Mark all format area block in the bm_used bitmap. */
-	if (reiser4_fs_layout(control->repair->fs, callback_format_mark,
-			      filter->bm_used)) 
-	{
+	if (reiser4_fs_layout(fs, cb_format_mark, filter->bm_used)) {
 		aal_error("Failed to mark the filesystem area as "
 			  "used in the bitmap.");
 		return -EINVAL;
 	}
 	
-	control->repair->fs->alloc->hook.alloc = callback_alloc;
-	control->repair->fs->alloc->hook.release = callback_release;
-	control->repair->fs->alloc->hook.data = control;
+	fs->alloc->hook.alloc = cb_alloc;
+	fs->alloc->hook.release = cb_release;
+	fs->alloc->hook.data = control;
 	
 	/* Allocate a bitmap of twig blocks in the tree. */
 	if (!(control->bm_twig = filter->bm_twig = aux_bitmap_create(fs_len))) {
-		aal_error("Failed to allocate a bitmap of twig "
-			  "blocks.");
+		aal_error("Failed to allocate a bitmap of twig blocks.");
 		return -EINVAL;
 	}
 
@@ -192,9 +196,7 @@ static errno_t repair_filter_prepare(repair_control_t *control,
 
 /* Mark blk in the scan bitmap if not marked in used bitmap (in this case it 
    is a node). */
-static errno_t callback_region_mark(void *object, blk_t blk, 
-				    uint64_t count, void *data)
-{
+static errno_t cb_region_mark(blk_t blk, uint64_t count, void *data) {
 	repair_control_t *control = (repair_control_t *)data;
 	uint32_t i;
 	
@@ -231,9 +233,9 @@ static errno_t repair_ds_prepare(repair_control_t *control, repair_ds_t *ds) {
 	ds->bm_leaf = control->bm_leaf;
 	ds->bm_twig = control->bm_twig;
 	ds->bm_met = control->bm_met;
-	ds->check_node = &control->check_node;
-	
-	ds->progress_handler = control->repair->progress_handler;
+	ds->stat.files = &control->files;
+	ds->mkidok = control->mkidok;
+	ds->mkid = control->mkid;
 	
 	repair = ds->repair;
 	
@@ -244,7 +246,7 @@ static errno_t repair_ds_prepare(repair_control_t *control, repair_ds_t *ds) {
 		file = fopen(control->repair->bitmap_file, "r");
 		
 		if (file == NULL) {
-			aal_fatal("Cannot not open the bitmap file (%s).",
+			aal_error("Cannot not open the bitmap file (%s).",
 				  control->repair->bitmap_file);
 			return -EINVAL;
 		}
@@ -254,6 +256,7 @@ static errno_t repair_ds_prepare(repair_control_t *control, repair_ds_t *ds) {
 		if (!(ds->bm_scan = aux_bitmap_unpack(&stream))) {
 			aal_error("Can't unpack the bitmap of "
 				  "packed blocks.");
+			
 			fclose(file);
 			aal_stream_fini(&stream);
 			return -EINVAL;
@@ -275,28 +278,33 @@ static errno_t repair_ds_prepare(repair_control_t *control, repair_ds_t *ds) {
 			ds->bm_scan->map[i] &= ~control->bm_met->map[i];
 		
 		goto fini;
-	}
-	
-	/* Allocate a bitmap of blocks to be scanned on this pass. */ 
-	if (!(ds->bm_scan = control->bm_scan = aux_bitmap_create(fs_len))) {
-		aal_error("Failed to allocate a bitmap of blocks "
-			  "unconnected from the tree.");
-		return -EINVAL;
+	} else {
+		/* Allocate a bitmap of blocks to be scanned on this pass. */ 
+		if (!(ds->bm_scan = control->bm_scan = aux_bitmap_create(fs_len))) {
+			aal_error("Failed to allocate a bitmap of blocks "
+				  "unconnected from the tree.");
+			return -EINVAL;
+		}
 	}
 
 	if (!(control->bm_alloc = aux_bitmap_create(fs_len))) {
-		aal_error("Failed to allocate a bitmap of allocated "
-			  "blocks.");
+		aal_error("Failed to allocate a bitmap of allocated blocks.");
 		return -EINVAL;
 	}
 
-	if ((res = reiser4_alloc_extract(repair->fs->alloc, control->bm_alloc)))
-		return res;
+	if (control->repair->flags & (1 << REPAIR_WHOLE)) {
+		aux_bitmap_invert(control->bm_alloc);
+	} else {
+		if ((res = reiser4_alloc_extract(repair->fs->alloc, 
+						 control->bm_alloc)))
+		{
+			return res;
+		}
+	}
 	
 	/* Mark all broken regions of allocator as to be scanned. */
 	if ((res = repair_alloc_layout_bad(repair->fs->alloc,
-					   callback_region_mark,
-					   control)))
+					   cb_region_mark, control)))
 		return res;
 
 	/* Build a bitmap of what was met already. */
@@ -319,7 +327,7 @@ static errno_t repair_ds_prepare(repair_control_t *control, repair_ds_t *ds) {
 		   has not been synced on disk. Scan through all its blocks. */
 		if (~control->bm_alloc->map[i] & control->bm_met->map[i]) {
 			reiser4_alloc_region(repair->fs->alloc, i * 8,
-					     callback_region_mark, control);
+					     cb_region_mark, control);
 		} else {
 			control->bm_scan->map[i] |= 
 				(control->bm_alloc->map[i] & 
@@ -361,8 +369,6 @@ static errno_t repair_ts_prepare(repair_control_t *control, repair_ts_t *ts,
 	ts->bm_twig = control->bm_twig;
 	ts->bm_met = control->bm_met;
 	
-	ts->progress_handler = control->repair->progress_handler;
-	
 	if (control->bm_scan) {
 		/* If this is the twig scan that goes after disk_scan, 
 		   close scan bitmap. */
@@ -387,14 +393,12 @@ static errno_t repair_am_prepare(repair_control_t *control, repair_am_t *am) {
 	aal_assert("vpf-861", control->repair->fs != NULL);
 	
 	aal_memset(am, 0, sizeof(*am));
-	
 	am->repair = control->repair;
 	am->bm_leaf = control->bm_leaf;
 	am->bm_twig = control->bm_twig;
 	am->bm_used = control->bm_used;
-	
-	am->progress_handler = control->repair->progress_handler;
-	
+	am->stat.files = &control->files;
+
 	for (i = 0; i < control->bm_met->size; i++) {
 		/* Leave there twigs and leaves that are not in the tree. */
 		control->bm_twig->map[i] &= ~(control->bm_used->map[i]);
@@ -402,7 +406,14 @@ static errno_t repair_am_prepare(repair_control_t *control, repair_am_t *am) {
 	}
 	
 	/* Assign the met bitmap to the block allocator. */
+	aux_bitmap_calc_marked(control->bm_met);
 	reiser4_alloc_assign(control->repair->fs->alloc, control->bm_met);
+
+	/* Set the amount of used blocks, allow to allocate blocks on the 
+	   add missing pass. */
+	reiser4_format_set_free(control->repair->fs->format, 
+				control->bm_met->total - 
+				control->bm_met->marked);
 	
 	aux_bitmap_close(control->bm_met);
 	
@@ -425,13 +436,13 @@ static errno_t repair_sem_prepare(repair_control_t *control,
 	aal_memset(sem, 0, sizeof(*sem));
 	
 	sem->repair = control->repair;
-	sem->progress_handler = control->repair->progress_handler;
 	
 	aal_assert("vpf-1335", control->repair->mode != RM_BUILD ||
 		   !aux_bitmap_marked(control->bm_twig));
 
 	aux_bitmap_close(control->bm_twig);
 	control->bm_twig = NULL;
+	sem->stat.files = control->files;
 	
 	if (control->repair->mode == RM_BUILD) {
 		aal_assert("vpf-1335", control->repair->mode != RM_BUILD || 
@@ -443,6 +454,12 @@ static errno_t repair_sem_prepare(repair_control_t *control,
 		/* Assign the used bitmap to the block allocator. */
 		reiser4_alloc_assign(control->repair->fs->alloc, control->bm_used);
 		reiser4_alloc_sync(control->repair->fs->alloc);
+	
+		/* Set the amount of used blocks, allow to allocate blocks on 
+		   the semantic pass */
+		reiser4_format_set_free(control->repair->fs->format,
+					control->bm_used->total - 
+					control->bm_used->marked);
 
 		aux_bitmap_close(control->bm_used);
 		control->bm_used = NULL;
@@ -469,10 +486,10 @@ static errno_t repair_sem_prepare(repair_control_t *control,
 
 		if (control->repair->mode == RM_CHECK)
 			return 0;
-		
+	
 		if (!(bm_temp = aux_bitmap_clone(control->bm_alloc))) {
-			aal_error("Failed to allocate a backup of "
-				  "allocated blocks bitmap.");
+			aal_error("Failed to allocate a bitmap "
+				  "for allocated blocks.");
 			return -EINVAL;
 		}
 		
@@ -496,6 +513,7 @@ static errno_t repair_sem_fini(repair_control_t *control,
 	uint64_t fs_len;
 
 	control->oid = sem->stat.oid + 1;
+	control->files = sem->stat.reached_files;
 	
 	/* In the BUILD mode alloc was built on bm_used, nothing to do. */
 	if (control->repair->mode == RM_BUILD)
@@ -504,7 +522,7 @@ static errno_t repair_sem_fini(repair_control_t *control,
 	fs_len = reiser4_format_get_len(control->repair->fs->format);
 	
 	if (repair_bitmap_compare(control->bm_alloc, control->bm_used, 0)) {
-		aal_error("On-disk used blocks and really used blocks "
+		fsck_mess("On-disk used blocks and really used blocks "
 			  "differ.%s", control->repair->mode == RM_FIX &&
 			  !control->repair->fatal ? " Fixed." : "");
 
@@ -541,10 +559,8 @@ static errno_t repair_cleanup_prepare(repair_control_t *control,
 	aal_assert("vpf-861", control->repair->fs != NULL);
 	
 	aal_memset(cleanup, 0, sizeof(*cleanup));
-	
 	cleanup->repair = control->repair;    
-	cleanup->progress_handler = control->repair->progress_handler;
-	
+
 	return 0;
 }
 
@@ -576,8 +592,6 @@ static errno_t debug_am_prepare(repair_control_t *control, repair_am_t *am) {
 		return -EINVAL;
 	}
 	
-	am->progress_handler = control->repair->progress_handler;
-	
 	aux_bitmap_calc_marked(am->bm_twig);
 	aux_bitmap_calc_marked(am->bm_leaf);
 	
@@ -601,9 +615,9 @@ static errno_t repair_update(repair_control_t *control) {
 	
 	if (correct != val) {
 		if (mode != RM_BUILD) {
-			aal_mess("Free block count %llu found in the format is "
-				 "wrong. %s %llu.", val, mode == RM_CHECK ? 
-				 "Sould be" : "Fixed to", correct);
+			fsck_mess("Free block count %llu found in the format is "
+				  "wrong. %s %llu.", val, mode == RM_CHECK ? 
+				  "Should be" : "Fixed to", correct);
 		}
 
 		if (mode != RM_CHECK)
@@ -619,21 +633,46 @@ static errno_t repair_update(repair_control_t *control) {
 	   shared oid handling will be realy. */
 	if (control->oid && control->oid > val) {
 		if (mode != RM_BUILD) {
-			aal_mess("First not used oid %llu is wrong. %s %llu.",
-				 val, mode == RM_CHECK ? "Sould be" : 
-				 "Fixed to", control->oid);
+			fsck_mess("First not used oid %llu is wrong. %s %llu.",
+				  val, mode == RM_CHECK ? "Should be" : 
+				  "Fixed to", control->oid);
 		}
 
 		if (mode != RM_CHECK) {
-			plug_call(fs->oid->entity->plug->o.oid_ops, set_next,
-				  fs->oid->entity, control->oid);
+			plug_call(fs->oid->ent->plug->o.oid_ops, set_next,
+				  fs->oid->ent, control->oid);
 		} else {
 			control->repair->fixable++;
 		}
 	}
 	
+	if (mode == RM_BUILD) {
+		/* Check the file count. Some files could be accounted more than
+		   once not in the BUILD mode. */
+		val = reiser4_oid_used(fs->oid);
+
+		if (control->files && control->files != val) {
+			fsck_mess("File count %llu is wrong. %s %llu.",
+				  val, mode != RM_BUILD ? "Should be" : 
+				  "Fixed to", control->files);
+
+			if (mode == RM_BUILD) {
+				plug_call(fs->oid->ent->plug->o.oid_ops, set_used,
+					  fs->oid->ent, control->files);
+			}
+
+			/* This is not a problem to live with wrong file count.
+			   
+			else {
+			   control->repair->fixable++;
+			}
+
+			*/
+		}
+	}
+
 	/* The tree height should be set correctly at the filter pass. */
-	/* FIXME: File count is not ready. What about flushes, mkfs_id? */
+	/* FIXME: What about flushes? */
 	
 	return 0;
 }
@@ -676,7 +715,7 @@ errno_t repair_check(repair_data_t *repair) {
 	
 	control.repair = repair;
 	
-	if (repair->debug_flag) {
+	if (repair->flags & (1 << REPAIR_DEBUG)) {
 		/* Debugging */
 		if ((res = debug_am_prepare(&control, &am)))
 			goto error;
@@ -729,32 +768,36 @@ errno_t repair_check(repair_data_t *repair) {
 		repair_ts_fini(&control);
 	}
 
-	if (repair->mode != RM_BUILD && repair->fatal) {
-		aal_mess("\nFatal corruptions were found. "
+	if (repair->fatal) {
+		aal_warn("Fatal corruptions were found. "
 			 "Semantic pass is skipped.");
-	} else {
-		/* Check the semantic reiser4 tree. */
-		if ((res = repair_sem_prepare(&control, &sem)))
-			goto error;
-
-		if ((res = repair_semantic(&sem)))
-			goto error;
-
-		if ((res = repair_sem_fini(&control, &sem)))
-			goto error;
-	}
-		
-	if (repair->mode == RM_BUILD && !repair->fatal) {
-		/* Throw the garbage away. */
-		if ((res = repair_cleanup_prepare(&control, &cleanup)))
-			goto error;
-		
-		if ((res = repair_cleanup(&cleanup)))
-			goto error;
-	}
+		goto update;
+	} 
 	
+	/* Check the semantic reiser4 tree. */
+	if ((res = repair_sem_prepare(&control, &sem)))
+		goto error;
+
+	if ((res = repair_semantic(&sem)))
+		goto error;
+
+	if ((res = repair_sem_fini(&control, &sem)))
+		goto error;
+
+	if (repair->mode != RM_BUILD || repair->fatal) 
+		goto update;
+
+	/* Throw the garbage away. */
+	if ((res = repair_cleanup_prepare(&control, &cleanup)))
+		goto error;
+
+	if ((res = repair_cleanup(&cleanup)))
+		goto error;
+
+	
+ update:
 	/* Update SB data */
-	if ((res = repair_update(&control))) 
+	if (!repair->fatal && (res = repair_update(&control))) 
 		goto error;
 	
  error:
