@@ -3,6 +3,7 @@
    
    dot_o.c -- ".o" fibration code. */
 
+#ifdef ENABLE_DOT_O_FIBRE
 #include <reiser4/plugin.h>
 
 static uint8_t fibre_dot_o_build(char *name, uint32_t len) {
@@ -21,9 +22,9 @@ static reiser4_fibre_ops_t fibre_dot_o_ops = {
 static reiser4_plug_t fibre_dot_o_plug = {
 	.cl    = class_init,
 	.id    = {FIBRE_DOT_O_ID, 0, FIBRE_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "dot_o_fibre",
-	.desc  = "'.o' fibration plugin for reiser4, ver. " VERSION,
+	.desc  = "'.o' fibration plugin for reiser4. ",
 #endif
 	.o = {
 		.fibre_ops = &fibre_dot_o_ops
@@ -35,3 +36,4 @@ static reiser4_plug_t *fibre_dot_o_start(reiser4_core_t *c) {
 }
 
 plug_register(fibre_dot_o, fibre_dot_o_start, NULL);
+#endif
