@@ -22,9 +22,9 @@ static reiser4_fibre_ops_t fibre_ext_3_ops = {
 static reiser4_plug_t fibre_ext_3_plug = {
 	.cl    = class_init,
 	.id    = {FIBRE_EXT_3_ID, 0, FIBRE_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "ext_3_fibre",
-	.desc  = "Lexicographic fibration plugin for reiser4, ver. " VERSION,
+	.desc  = "Lexicographic fibration plugin for reiser4. ",
 #endif
 	.o = {
 		.fibre_ops = &fibre_ext_3_ops
