@@ -7,7 +7,7 @@
 #ifdef ENABLE_DEG_HASH
 #include <reiser4/plugin.h>
 
-uint64_t deg_hash_build(char *name, uint32_t len) {
+uint64_t deg_hash_build(unsigned char *name, uint32_t len) {
 	return 0xc0c0c0c010101010ull;
 }
 
@@ -18,9 +18,9 @@ static reiser4_hash_ops_t deg_hash_ops = {
 static reiser4_plug_t deg_hash_plug = {
 	.cl = class_init,
 	.id = {HASH_DEG_ID, 0, HASH_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "deg_hash",
-	.desc  = "Degenerate hash for reiser4, ver. " VERSION,
+	.desc  = "Degenerate hash for reiser4. ",
 #endif
 	.o = {
 		.hash_ops = &deg_hash_ops
