@@ -6,7 +6,7 @@
 #ifdef ENABLE_RUPASOV_HASH
 #include <reiser4/plugin.h>
 
-uint64_t rupasov_hash_build(char *name, uint32_t len) {
+uint64_t rupasov_hash_build(unsigned char *name, uint32_t len) {
 	uint32_t i;
 	uint64_t a, c;
 	uint32_t j, pow;
@@ -56,9 +56,9 @@ static reiser4_hash_ops_t rupasov_hash_ops = {
 static reiser4_plug_t rupasov_hash_plug = {
 	.cl    = class_init,
 	.id    = {HASH_RUPASOV_ID, 0, HASH_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "rupasov_hash",
-	.desc  = "Rupasov hash for reiser4, ver. " VERSION,
+	.desc  = "Rupasov hash for reiser4. ",
 #endif
 	.o = {
 		.hash_ops = &rupasov_hash_ops
