@@ -24,7 +24,7 @@
    10)Remove the space between the end of the last entry and the end of 
    the item. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 
 #include "cde40.h"
 #include <aux/bitmap.h>
@@ -43,7 +43,7 @@
 #define UNIT_LEN_MIN(kind, pol)		\
         (en_size(pol) + ENTRY_LEN_MIN(kind, pol))
 
-#define DENTRY_LEN_MIN			\
+#define DIRITEM_LEN_MIN			\
         (UNIT_LEN_MIN(S_NAME) + sizeof(cde40_t))
 
 #define en_len_min(count, pol)		\
@@ -54,7 +54,7 @@
 
 struct entry_flags {
 	uint8_t *elem;
-	uint8_t count;
+	uint32_t count;
 };
     
 /* Extention for repair_flag_t */
@@ -156,7 +156,7 @@ static uint32_t cde40_name_len(char *body, uint32_t start, uint32_t end) {
 }
 
 /* Returns amount of entries detected. */
-static uint8_t cde40_short_entry_detect(reiser4_place_t *place, 
+static uint32_t cde40_short_entry_detect(reiser4_place_t *place, 
 					uint32_t start_pos, 
 					uint32_t length, 
 					uint8_t mode)
@@ -176,13 +176,15 @@ static uint8_t cde40_short_entry_detect(reiser4_place_t *place,
 	if (mode == REPAIR_SKIP)
 		return length / ENTRY_LEN_MIN(S_NAME, pol);
 	
+	start_pos++;
 	for (offset = ENTRY_LEN_MIN(S_NAME, pol); offset < length; 
 	     offset += ENTRY_LEN_MIN(S_NAME, pol), start_pos++) 
 	{
-		aal_error("Node (%llu), item (%u), unit (%u): unit "
+		fsck_mess("Node (%llu), item (%u), unit (%u), [%s]: unit "
 			  "offset (%u) is wrong, should be (%u). %s", 
-			  place->node->block->nr, place->pos.item,
-			  start_pos, cde_get_offset(place, start_pos, pol),
+			  place_blknr(place), place->pos.item, start_pos,
+			  print_key(cde40_core, &place->key),
+			  cde_get_offset(place, start_pos, pol),
 			  limit + offset,  mode == RM_BUILD ? "Fixed." : "");
 		
 		if (mode == RM_BUILD)
@@ -193,7 +195,7 @@ static uint8_t cde40_short_entry_detect(reiser4_place_t *place,
 }
 
 /* Returns amount of entries detected. */
-static uint8_t cde40_long_entry_detect(reiser4_place_t *place, 
+static uint32_t cde40_long_entry_detect(reiser4_place_t *place, 
 				       uint32_t start_pos, 
 				       uint32_t length, 
 				       uint8_t mode)
@@ -234,12 +236,13 @@ static uint8_t cde40_long_entry_detect(reiser4_place_t *place,
 		if (mode != REPAIR_SKIP && 
 		    l_limit != cde_get_offset(place, start_pos + count, pol)) 
 		{
-			aal_error("Node %llu, item %u, unit (%u): unit "
-				  "offset (%u) is wrong, should be (%u). "
-				  "%s", place->node->block->nr, place->pos.item,
+			fsck_mess("Node (%llu), item (%u), unit (%u), [%s]: "
+				  "unit offset (%u) is wrong, should be (%u)."
+				  "%s", place_blknr(place), place->pos.item,
 				  start_pos + count,
+				  print_key(cde40_core, &place->key), 
 				  cde_get_offset(place, start_pos + count, pol),
-				  l_limit, mode == RM_BUILD ? "Fixed." : "");
+				  l_limit, mode == RM_BUILD ? " Fixed." : "");
 			
 			if (mode == RM_BUILD) {
 				cde_set_offset(place, start_pos + count, 
@@ -251,11 +254,11 @@ static uint8_t cde40_long_entry_detect(reiser4_place_t *place,
 	return l_limit == r_limit ? count : 0;
 }
 
-static inline uint8_t cde40_entry_detect(reiser4_place_t *place, uint32_t start_pos,
+static inline uint32_t cde40_entry_detect(reiser4_place_t *place, uint32_t start_pos,
 					 uint32_t end_pos, uint8_t mode)
 {
 	uint32_t pol;
-	uint8_t count;
+	uint32_t count;
 
 	pol = cde40_key_pol(place);
 	count = cde40_short_entry_detect(place, start_pos,
@@ -307,10 +310,11 @@ static errno_t cde40_offsets_range_check(reiser4_place_t *place,
 	for (i = 0; i < flags->count; i++) {
 		/* Check if the offset is valid. */
 		if (cde40_offset_check(place, i)) {
-			aal_error("Node %llu, item %u, unit %u: unit "
-				  "offset (%u) is wrong.", 
-				  place->node->block->nr, place->pos.item, 
-				  i, cde_get_offset(place, i, pol));
+			fsck_mess("Node (%llu), item (%u), unit (%u), "
+				  "[%s]: unit offset (%u) is wrong.",
+				  place_blknr(place), place->pos.item, i,
+				  print_key(cde40_core, &place->key),
+				  cde_get_offset(place, i, pol));
 			
 			/* mark offset wrong. */	    
 			aal_set_bit(flags->elem + i, NR);
@@ -385,13 +389,23 @@ static errno_t cde40_offsets_range_check(reiser4_place_t *place,
 	return res;
 }
 
-static errno_t cde40_filter(reiser4_place_t *place, struct entry_flags *flags,
-			    uint8_t mode)
+static errno_t cde40_filter(reiser4_place_t *place, 
+			    struct entry_flags *flags,
+			    repair_hint_t *hint)
 {
-	uint32_t i, last;
+	/* The correct number of units. */
+	uint32_t real_count;
+	/* Estimated number of units. The maximim possible limit. */
 	uint32_t e_count;
+	/* Offset within the item. */
 	uint32_t offset;
 	errno_t res = 0;
+	/* The first relable unit offset. */
+	uint32_t first;
+	/* The last relable unit offset. */
+	uint32_t last;
+	/* Count of bytes to be removed at the end of the item. */
+	uint32_t tail;
 	uint32_t pol;
 	
 	aal_assert("vpf-757", flags != NULL);
@@ -405,9 +419,10 @@ static errno_t cde40_filter(reiser4_place_t *place, struct entry_flags *flags,
 	
 	if (last == 0) {
 		/* No one R unit was found */
-		aal_error("Node %llu, item %u: no one valid unit has "
-			  "been found. Does not look like a valid `%s` "
-			  "item.", place->node->block->nr, place->pos.item, 
+		fsck_mess("Node (%llu), item (%u), [%s]: no one valid unit "
+			  "has been found. Does not look like a valid `%s` "
+			  "item.", place_blknr(place), place->pos.item, 
+			  print_key(cde40_core, &place->key),
 			  place->plug->label);
 		
 		return RE_FATAL;
@@ -416,7 +431,8 @@ static errno_t cde40_filter(reiser4_place_t *place, struct entry_flags *flags,
 	/* the last unit size is not checked yet, so save - 1 into @count. */
 	flags->count = --last;
 	
-	offset = cde_get_offset(place, last, pol) + ob_size(pol);
+	tail = cde_get_offset(place, last, pol);
+	offset = tail + ob_size(pol);
 	
 	/* Last is the last valid offset. If the last unit is valid also, count
 	   is the last + 1. */
@@ -428,135 +444,168 @@ static errno_t cde40_filter(reiser4_place_t *place, struct entry_flags *flags,
 		offset += aal_strlen(place->body + offset) + 1;
 
 		/* If nothing to be removed, the last unit is relable. */
-		if (offset == place->len) 
+		if (offset == place->len)
 			flags->count++;
 	} else {
 		aal_bug("vpf-1560", "Amount of detected units exceeds the "
 			"item length.");
 	}
 	
+	/* Amount of bytes that should be removed at the end. */
+	if (flags->count == last) {
+		tail = place->len - tail;
+	} else {
+		tail = 0;
+	}
+
 	/* Count is the amount of recovered elements. */
 	
 	/* Find the first relable. */
-	for (i = 0; i < flags->count && !aal_test_bit(flags->elem + i, R); i++) {}
+	for (first = 0; 
+	     first < flags->count && 
+	     !aal_test_bit(flags->elem + first, R);
+	     first++) {}
 	
-	/* Estimate the amount of units on the base of the first R element. */
-	e_count = cde40_count_estimate(place, i);
+	/* Estimate the amount of unit headers by the first R element offset. */
+	e_count = cde40_count_estimate(place, first);
 	
-	/* Estimated count must be less then count found on the base of the last 
-	 * valid offset. */
+	/* Estimated count must be larger than the recovered count. */
 	aal_assert("vpf-765", e_count >= flags->count);
 	
-	if (last != flags->count && e_count > flags->count && mode == RM_BUILD) {
-		/* If there is enough space for another entry header, set 
-		   @count unit offset to the item length to remove item 
-		   headers correctly later. */
-		cde_set_offset(place, flags->count, place->len, pol);
-	} 
+	/* If the first unit offset is correct, the correct unit number 
+	   is estimated count, otherwise will be changed later. */
+	real_count = e_count;
 	
-	if (i) {
-		/* Some first offset are not relable. Consider count as 
-		   the correct count and set the first offset just after 
-		   the last unit.*/
-		if (mode == RM_BUILD) {
+	if (first) {
+		/* Some first offset are not relable. Consider count as
+		   the correct count and set the first offset just after
+		   the last unit to remove first items correctly later.*/
+		if (hint->mode == RM_BUILD) {
 			cde_set_offset(place, 0, sizeof(cde40_t) + 
-				   en_size(pol) * flags->count, pol);
-			place_mkdirty(place);
+				       en_size(pol) * flags->count, pol);
 		}
-	}
-	
-	if (e_count != cde_get_units(place)) {
-		aal_error("Node %llu, item %u: unit count (%u) "
-			  "is not correct. Should be (%u). %s",
-			  place->node->block->nr,  place->pos.item,
-			  cde_get_units(place), e_count, 
-			  mode == RM_CHECK ? "" : "Fixed.");
 		
-		if (mode == RM_CHECK) {
+		/* There are some not correct units at the beginning, 
+		   the correct unit number is the recovered count. */
+		real_count = flags->count;
+	} else if (!tail && e_count > flags->count) {
+		/* If there is enough space for another entry header 
+		   (e_count > count), and nothing to remove at the end 
+		   of the item, the last unit is correct (tail == 0),
+		   set @count unit offset to the item length to remove 
+		   redundant item headers correctly later. */
+		if (hint->mode == RM_BUILD) {
+			cde_set_offset(place, flags->count, place->len, pol);
+		}
+	} 
+
+	/* Fix the count before removing anything. */
+	if (real_count != cde_get_units(place)) {
+		fsck_mess("Node (%llu), item (%u), [%s]: unit count "
+			  "(%u) is not correct. Should be (%u).%s",
+			  place_blknr(place), place->pos.item,
+			  print_key(cde40_core, &place->key), 
+			  cde_get_units(place), real_count, 
+			  hint->mode == RM_CHECK ? "" : 
+			  " Fixed.");
+		
+		if (hint->mode == RM_CHECK) {
 			res |= RE_FIXABLE;
 		} else {
-			cde_set_units(place, e_count);
+			cde_set_units(place, real_count);
 			place_mkdirty(place);
 		}
 	}
 	
-	if (flags->count != e_count) {
-		/* Estimated count is greater then the recovered count, in other 
+	if (flags->count != real_count) {
+		/* Estimated count is greater then the recovered count, in other
 		   words there are some last unit headers should be removed. */
-		aal_error("Node %llu, item %u: entries [%u..%u] look "
-			  "corrupted. %s", place->node->block->nr,
-			  place->pos.item, flags->count, e_count - 1, 
-			  mode == RM_BUILD ? "Removed." : "");
+		fsck_mess("Node (%llu), item (%u), [%s]: entries [%u..%u] look "
+			  "corrupted. %s", place_blknr(place), place->pos.item,
+			  print_key(cde40_core, &place->key), flags->count, 
+			  real_count - 1, hint->mode == RM_BUILD ? 
+			  "Removed." : "");
 		
-		if (mode == RM_BUILD) {
-			place->len -= cde40_cut(place, flags->count, 
-						e_count - flags->count, 
-						place->len);
+		if (hint->mode == RM_BUILD) {
+			hint->len += cde40_cut(place, flags->count, 
+					       real_count - flags->count,
+					       place->len);
 			
 			place_mkdirty(place);
 		} else {
 			res |= RE_FATAL;
 		}
+	} else if (tail) {
+		/* Count of recovered units metches the estimated count 
+		   but there are some btes at the end to be removed. */
+		fsck_mess("Node (%llu), item (%u), [%s]: %u redundant bytes at "
+			  "the end.%s", place_blknr(place), place->pos.item,
+			  print_key(cde40_core, &place->key), tail, 
+			  hint->mode == RM_BUILD ? "Removed." : "");
+
+		if (hint->mode == RM_BUILD)
+			hint->len += tail;
 	}
 	
-	if (i) {
+	if (first) {
 		/* Some first units should be removed. */
-		aal_error("Node %llu, item %u: entries [%u..%u] look "
-			  " corrupted. %s", place->node->block->nr, 
-			  place->pos.item, 0, i - 1, 
-			  mode == RM_BUILD ? "Removed." : "");
+		fsck_mess("Node (%llu), item (%u), [%s]: entries [%u..%u] look "
+			  "corrupted. %s", place_blknr(place), place->pos.item,
+			  print_key(cde40_core, &place->key), 0, first - 1, 
+			  hint->mode == RM_BUILD ? "Removed." : "");
 		
-		if (mode == RM_BUILD) {
-			place->len -= cde40_cut(place, 0, i, place->len);
+		if (hint->mode == RM_BUILD) {
+			hint->len += cde40_cut(place, 0, first, place->len);
 
 			place_mkdirty(place);
 			
-			aal_memmove(flags->elem, flags->elem + i, 
-				    flags->count - i);
+			aal_memmove(flags->elem, flags->elem + first, 
+				    flags->count - first);
 			
-			flags->count -= i;
-			i = 0;
+			flags->count -= first;
+			first = 0;
 		} else {
 			res |= RE_FATAL;
 		}
 	}
 	
-	/* Units before @i and after @count were handled, do not care about them 
+	/* Units before @first and after @count were handled, do not care about them 
 	   anymore. Handle all not relable units between them. */
 	last = MAX_UINT32;
-	for (; i < flags->count; i++) {
+	for (; first < flags->count; first++) {
 		if (last == MAX_UINT32) {
 			/* Looking for the problem interval start. */
-			if (!aal_test_bit(flags->elem + i, R))
-				last = i - 1;
+			if (!aal_test_bit(flags->elem + first, R))
+				last = first - 1;
 			
 			continue;
 		}
 		
 		/* Looking for the problem interval end. */
-		if (aal_test_bit(flags->elem + i, R)) {
-			aal_error("Node %llu, item %u: entries "
-				  "[%u..%u] look corrupted. %s", 
-				  place->node->block->nr, place->pos.item,
-				  last, i - 1, mode == RM_BUILD ? 
+		if (aal_test_bit(flags->elem + first, R)) {
+			fsck_mess("Node (%llu), item (%u), [%s]: "
+				  "entries [%u..%u] look corrupted. %s",
+				  place_blknr(place), place->pos.item, 
+				  print_key(cde40_core, &place->key), 
+				  last, first - 1, hint->mode == RM_BUILD ? 
 				  "Removed." : "");
 
-			if (mode != RM_BUILD) {
+			if (hint->mode != RM_BUILD) {
 				res |= RE_FATAL;
 				last = MAX_UINT32;
 				continue;
 			}
 			
-			place->len -= cde40_cut(place, last, i - last, 
-						place->len);
+			hint->len += cde40_cut(place, last, first - last, 
+					       place->len);
 
 			place_mkdirty(place);
 			
-			aal_memmove(flags->elem + last, flags->elem + i,
-				    flags->count - i);
+			aal_memmove(flags->elem + last, flags->elem + first,
+				    flags->count - first);
 
-			flags->count -= (i - last);
-			i = last;
+			flags->count -= (first - last);
+			first = last;
 			last = MAX_UINT32;
 		}
 	}
@@ -564,21 +613,23 @@ static errno_t cde40_filter(reiser4_place_t *place, struct entry_flags *flags,
 	return res;
 }
 
-errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
+errno_t cde40_check_struct(reiser4_place_t *place, repair_hint_t *hint) {
 	static reiser4_key_t pkey, ckey;
 	struct entry_flags flags;
 	uint32_t pol;
 	errno_t res = 0;
-	int i;
+	uint32_t i;
 	
 	aal_assert("vpf-267", place != NULL);
+	aal_assert("vpf-1641", hint != NULL);
 
 	pol = cde40_key_pol(place);
 	
 	if (place->len < en_len_min(1, pol)) {
-		aal_error("Node %llu, item %u: item length (%u) is too "
-			  "small to contain a valid item.", 
-			  place->node->block->nr, place->pos.item, place->len);
+		fsck_mess("Node (%llu), item (%u), [%s]: item length "
+			  "(%u) is too small to contain a valid item.",
+			  place_blknr(place), place->pos.item, 
+			  print_key(cde40_core, &place->key), place->len);
 		return RE_FATAL;
 	}
 	
@@ -588,12 +639,12 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 	/* map consists of bit pairs - [not relable -R, relable - R] */
 	flags.elem = aal_calloc(flags.count, 0);
 	
-	res |= cde40_offsets_range_check(place, &flags, mode);
+	res |= cde40_offsets_range_check(place, &flags, hint->mode);
 	
 	if (res) goto error;
 	
 	/* Filter units with relable offsets from others. */
-	res |= cde40_filter(place, &flags, mode);
+	res |= cde40_filter(place, &flags, hint);
 
 	aal_free(flags.elem);
 	
@@ -605,12 +656,12 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 	   item is thrown away if smth wrong, to be improved later. */
 	for (i = 1; i <= flags.count; i++) {
 		reiser4_key_t key;
-		trans_hint_t hint;
+		trans_hint_t trans;
 		uint32_t offset;
 		
 		cde40_get_hash(place, i - 1, &key);
 
-		offset = i == flags.count ? place->len : 
+		offset = i == flags.count ? place->len - hint->len: 
 			cde_get_offset(place, i, pol);
 		
 		if (cde_get_offset(place, i - 1, pol) + ob_size(pol) == offset){
@@ -620,24 +671,25 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 				continue;
 
 			/* Hashed, key is wrong, remove the entry. */
-			aal_error("Node (%llu), item (%u): wrong key "
+			fsck_mess("Node (%llu), item (%u): wrong key "
 				  "[%s] of the unit (%u).%s", 
-				  place->node->block->nr, place->pos.item,
-				  cde40_core->key_ops.print(&key, PO_INODE),
-				  i - 1, mode == RM_BUILD ? " Removed." : "");
+				  place_blknr(place), place->pos.item,
+				  print_inode(cde40_core, &key),
+				  i - 1, hint->mode == RM_BUILD ? 
+				  " Removed." : "");
 			
-			if (mode != RM_BUILD) {
+			if (hint->mode != RM_BUILD) {
 				res |= RE_FATAL;
 				continue;
 			}
 
 			/* Remove the entry. */
-			hint.count = 1;
+			trans.count = 1;
 
-			if ((res |= cde40_delete(place, i - 1, &hint)) < 0)
+			if ((res |= cde40_delete(place, i - 1, &trans)) < 0)
 				return res;
 
-			place->len -= hint.len;
+			hint->len += trans.len;
 			flags.count--;
 			i--;
 			continue;
@@ -648,24 +700,25 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 				continue;
 			
 			/* Not hashed, key is wrong, remove the entry. */
-			aal_error("Node (%llu), item (%u): wrong key "
+			fsck_mess("Node (%llu), item (%u): wrong key "
 				  "[%s] of the unit (%u).%s", 
-				  place->node->block->nr, place->pos.item, 
-				  cde40_core->key_ops.print(&key, PO_INODE),
-				  i - 1, mode == RM_BUILD ? " Removed." : "");
+				  place_blknr(place), place->pos.item,
+				  print_inode(cde40_core, &key),
+				  i - 1, hint->mode == RM_BUILD ? 
+				  " Removed." : "");
 			
-			if (mode != RM_BUILD) {
+			if (hint->mode != RM_BUILD) {
 				res |= RE_FATAL;
 				continue;
 			}
 			
 			/* Remove the entry. */
-			hint.count = 1;
+			trans.count = 1;
 
-			if ((res |= cde40_delete(place, i - 1, &hint)) < 0)
+			if ((res |= cde40_delete(place, i - 1, &trans)) < 0)
 				return res;
 
-			place->len -= hint.len;
+			hint->len += trans.len;
 			flags.count--;
 			i--;
 			continue;
@@ -685,11 +738,10 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 		if (plug_call(pkey.plug->o.key_ops, compfull, 
 			      &pkey, &ckey) == 1) 
 		{
-			aal_error("Node (%llu), item (%u): wrong order of "
-				  "units {%d, %d}. The whole item to be "
-				  "removed -- will be improved soon.",
-				  place->node->block->nr, place->pos.item,
-				  i - 1, i);
+			fsck_mess("Node (%llu), item (%u), [%s]: wrong order "
+				  "of units [%d, %d]. The whole item is to be "
+				  "removed.",place_blknr(place),place->pos.item,
+				  print_key(cde40_core, &place->key), i - 1, i);
 			return res | RE_FATAL;
 		}
 		pkey = ckey;
@@ -700,12 +752,12 @@ errno_t cde40_check_struct(reiser4_place_t *place, uint8_t mode) {
 	
 	cde40_get_hash(place, 0, &ckey);
 	if (plug_call(ckey.plug->o.key_ops, compfull, &ckey, &place->key)) {
-		aal_error("Node (%llu), item (%u): the item key [%s] does "
+		fsck_mess("Node (%llu), item (%u): the item key [%s] does "
 			  "not match the first unit key [%s].%s", 
-			  place->node->block->nr, place->pos.item,
-			  cde40_core->key_ops.print(&place->key, PO_INODE),
-			  cde40_core->key_ops.print(&ckey, PO_INODE),
-			  mode == RM_BUILD ? " Fixed." : "");
+			  place_blknr(place), place->pos.item,
+			  print_inode(cde40_core, &place->key),
+			  print_inode(cde40_core, &ckey),
+			  hint->mode == RM_BUILD ? " Fixed." : "");
 		res |= RE_FATAL;
 	}
 	
@@ -749,15 +801,15 @@ static int cde40_comp_entry(reiser4_place_t *place1, uint32_t pos1,
 
 /* Estimate the space needed for the insertion of the not overlapped part 
    of the item, overlapped part does not need any space. */
-errno_t cde40_prep_merge(reiser4_place_t *place, trans_hint_t *hint) {
+errno_t cde40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *trans) {
 	uint32_t sunits, send;
 	uint32_t offset, pol;
 	reiser4_place_t *src;
 
 	aal_assert("vpf-957", place != NULL);
-	aal_assert("vpf-959", hint != NULL);
+	aal_assert("vpf-959", trans != NULL);
 
-	src = (reiser4_place_t *)hint->specific;
+	src = (reiser4_place_t *)trans->specific;
 	sunits = cde40_units(src);
 	pol = cde40_key_pol(place);
 
@@ -773,46 +825,46 @@ errno_t cde40_prep_merge(reiser4_place_t *place, trans_hint_t *hint) {
 	} else
 		send = sunits;
 
-	hint->bytes = 0;
-	hint->count = send - src->pos.unit;
+	trans->bytes = 0;
+	trans->count = send - src->pos.unit;
 	offset = send == sunits ? src->len : cde_get_offset(src, send, pol);
 	
 	/* Len to be inserted is the size of header + item bodies. */
-	hint->len = hint->count * en_size(pol) + offset -
+	trans->len = trans->count * en_size(pol) + offset -
 		cde_get_offset(src, src->pos.unit, pol);
 	
-	hint->overhead = (place->pos.unit == MAX_UINT32 && hint->len) ? 
-		cde40_overhead(place) : 0;
+	trans->overhead = cde40_overhead();
 
 	return 0;
 }
 
-int64_t cde40_merge(reiser4_place_t *place, trans_hint_t *hint) {
+int64_t cde40_insert_raw(reiser4_place_t *place, trans_hint_t *trans) {
 	uint32_t dpos, dunits;
 	uint32_t spos, sunits;
 	reiser4_place_t *src;
 	errno_t res;
 	
 	aal_assert("vpf-1370", place != NULL);
-	aal_assert("vpf-1371", hint != NULL);
+	aal_assert("vpf-1371", trans != NULL);
 
-	src = (reiser4_place_t *)hint->specific;
+	src = (reiser4_place_t *)trans->specific;
 	
 	sunits = cde40_units(src);
 	
-	if (hint->count) {
-		/* Expand @place & copy @hint->count units there from @src. */
+	if (trans->count) {
+		/* Expand @place & copy @trans->count units there from @src. */
 		dpos = place->pos.unit == MAX_UINT32 ? 0 : place->pos.unit;
 		
 		if (place->pos.unit != MAX_UINT32)
-			cde40_expand(place, dpos, hint->count, hint->len);
+			cde40_expand(place, dpos, trans->count, trans->len);
 		else
 			cde_set_units(place, 0);
 		
-		res = cde40_copy(place, dpos, src, src->pos.unit, hint->count);
-		if (res) return res;
+		if ((res = cde40_copy(place, dpos, src, src->pos.unit, 
+				      trans->count)))
+			return res;
 
-		spos = src->pos.unit + hint->count;
+		spos = src->pos.unit + trans->count;
 
 		place_mkdirty(place);
 	} else {
@@ -831,74 +883,88 @@ int64_t cde40_merge(reiser4_place_t *place, trans_hint_t *hint) {
 	}
 	
 	/* Update the item key. */
-	if (place->pos.unit == 0 && hint->count) {
+	if (place->pos.unit == 0 && trans->count) {
 		plug_call(place->key.plug->o.key_ops, assign,
-			  &place->key, &hint->offset);
+			  &place->key, &trans->offset);
 	}
 	
 	if (spos == sunits)
-		return cde40_maxposs_key(src, &hint->maxkey);
+		return cde40_maxposs_key(src, &trans->maxkey);
 	
-	return cde40_get_hash(src, spos, &hint->maxkey);
+	return cde40_get_hash(src, spos, &trans->maxkey);
 }
 
 #define PRINT_NAME_LIMIT 38
 
 /* Prints cde item into passed @stream */
 void cde40_print(reiser4_place_t *place, aal_stream_t *stream, uint16_t options) {
-	uint32_t pol;
-	uint32_t i, j;
+	uint32_t namewidth = 0;
+	uint64_t locality = 0;
+	uint64_t objectid = 0;
+	uint32_t i, j, count;
 	char name[256];
-	uint64_t locality;
-	uint64_t objectid;
-	uint32_t namewidth;
+	uint32_t pol;
 	
 	aal_assert("umka-548", place != NULL);
 	aal_assert("umka-549", stream != NULL);
 
-	aal_stream_format(stream, "UNITS=%u\n", cde_get_units(place));
 	aal_stream_format(stream, "NR  NAME%*s OFFSET HASH%*s "
-			  "SDKEY%*s\n", PRINT_NAME_LIMIT - 5, " ", 29, " ", 13, " ");
+			  "SDKEY%*s\n", PRINT_NAME_LIMIT - 5, " ", 29, 
+			  " ", 13, " ");
 	
 	pol = cde40_key_pol(place);
+	count = (place->len - sizeof(cde40_t)) / (en_size(pol));
+	if (count > cde_get_units(place))
+		count = cde_get_units(place);
 
 	/* Loop though the all entries and print them to @stream. */
-	for (i = 0; i < cde_get_units(place); i++) {
-		uint64_t offset, haobj;
-		void *objid = cde40_objid(place, i);
+	for (i = 0; i < count; i++) {
 		void *entry = cde40_entry(place, i);
-
-		if (objid >= place->body + place->len || 
-		    entry >= place->body + place->len)
-		{
-			aal_stream_format(stream, "Broken entry array "
-					  "detected.\n");
-			break;
-		}
+		uint64_t offset, haobj;
+		void *objid;
 		
-		cde40_get_name(place, i, name, sizeof(name));
+		
+		if (options != PO_UNIT_OFFSETS) {
+			objid = cde40_objid(place, i);
 
-		/* Cutting name by PRINT_NAME_LIMIT symbols */
-		if (aal_strlen(name) > PRINT_NAME_LIMIT) {
-			for (j = 0; j < 3; j++)
-				name[PRINT_NAME_LIMIT - 3 + j] = '.';
+			if (objid >= place->body + place->len || 
+			    entry >= place->body + place->len)
+			{
+				aal_stream_format(stream, "Broken entry array "
+						  "detected.\n");
+				break;
+			}
 
-			name[PRINT_NAME_LIMIT - 3 + j] = '\0';
+			cde40_get_name(place, i, name, sizeof(name));
+
+			/* Cutting name by PRINT_NAME_LIMIT symbols */
+			if (aal_strlen(name) > PRINT_NAME_LIMIT) {
+				for (j = 0; j < 3; j++)
+					name[PRINT_NAME_LIMIT - 3 + j] = '.';
+
+				name[PRINT_NAME_LIMIT - 3 + j] = '\0';
+			}
+
+			/* Getting locality, objectid. */
+			locality = ob_get_locality(objid, pol);
+			objectid = ob_get_objectid(objid, pol);
+			namewidth = PRINT_NAME_LIMIT - aal_strlen(name);
 		}
 
-		/* Getting locality, objectid. */
-		locality = ob_get_locality(objid, pol);
-		objectid = ob_get_objectid(objid, pol);
-		namewidth = PRINT_NAME_LIMIT - aal_strlen(name);
-
 		offset = ha_get_offset(entry, pol);
 		haobj = ha_get_objectid(entry, pol);
 
 		/* Putting data to @stream. */
-		aal_stream_format(stream, "%*d %s%*s %*u %.16llx:%.16llx "
-				  "%.7llx:%.7llx\n", 3, i, name, namewidth,
-				  " ", 5, en_get_offset(entry, pol), haobj,
-				  offset, locality, objectid);
+		if (options != PO_UNIT_OFFSETS) {
+			aal_stream_format(stream, "%*d %s%*s %*u %.16llx:%.16llx "
+					  "%.7llx:%.7llx\n", 3, i, name, namewidth,
+					  " ", 5, en_get_offset(entry, pol), haobj,
+					  offset, locality, objectid);
+		} else {
+			aal_stream_format(stream, "%*d %*u %.16llx:%.16llx\n", 
+					  3, i, 5, en_get_offset(entry, pol),
+					  haobj, offset);
+		}
 	}
 }
 
