@@ -125,8 +125,10 @@ static errno_t extent40_remove_units(reiser4_place_t *place, trans_hint_t *hint)
 	   levels know that some extent region is released and perform some
 	   actions like release blocks in block allocator, etc. */
 	if (hint->region_func) {
-		uint32_t i, start;
 		extent40_t *extent;
+		uint32_t start;
+		uint32_t width;
+		uint32_t i;
 
 		extent = extent40_body(place) + pos;
 			
@@ -134,14 +136,13 @@ static errno_t extent40_remove_units(reiser4_place_t *place, trans_hint_t *hint)
 			errno_t res;
 			
 			start = et40_get_start(extent);
+			width = et40_get_width(extent);
 
 			if (start == EXTENT_UNALLOC_UNIT || 
 			    start == EXTENT_HOLE_UNIT)
 				continue;
 			
-			if ((res = hint->region_func(place, start,
-						     et40_get_width(extent), 
-						     hint->data)))
+			if ((res = hint->region_func(start, width, hint->data)))
 				return res;
 		}
 	}
@@ -246,9 +247,11 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 		if (start != EXTENT_HOLE_UNIT && start != EXTENT_UNALLOC_UNIT) {
 			errno_t res;
 			
-			if ((res = hint->region_func(place, start, remove, 
+			if ((res = hint->region_func(start, remove, 
 						     hint->data)))
+			{
 				return res;
+			}
 		}
 
 		hint->bytes += remove * blksize;
@@ -516,7 +519,6 @@ static int64_t extent40_read_units(reiser4_place_t *place, trans_hint_t *hint) {
 	blksize = extent40_blksize(place);
 	secsize = extent40_secsize(place);
 
-
 	read_offset = plug_call(hint->offset.plug->o.key_ops,
 				get_offset, &hint->offset);
 	
@@ -754,27 +756,29 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 
 	uint64_t offset, bytes;
 	uint32_t blksize;
-	uint32_t size;
+	errno_t res;
 
 	device = extent40_device(place);
 	blksize = extent40_blksize(place);
 
 	/* Get offset aligned to the blksize. */
 	offset = (ins_offset - (ins_offset & (blksize - 1)));
+	count = (ins_offset + count + blksize - 1) / blksize * blksize - offset;
 
 	/* Prepare the key of the new allocated block. */
 	plug_call(place->key.plug->o.key_ops, assign, &key, &place->key);
 	plug_call(key.plug->o.key_ops, set_offset, &key, offset);
 
-	bytes = 0;
-
-	for (; count > 0; count -= size, ins_offset += size) {
+	bytes = count;
+	
+	if ((res = extent40_core->tree_ops.dec_free(place->node->tree, 
+						    count / blksize)))
+	{
+		return res;
+	}
+	
+	for (; count > 0; count -= blksize, offset += blksize) {
 		/* Calculating size to be written this time. */
-		size = blksize - (ins_offset % blksize);
-
-		if (size > count) 
-			size = count;
-
 		if (!(block = aal_block_alloc(device, blksize, 0)))
 			return -ENOMEM;
 
@@ -785,10 +789,7 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 
 		aal_hash_table_insert(blocks, ins_key, block);
 
-		bytes += blksize;
-
 		/* Update @key offset. */
-		offset += blksize;
 		plug_call(key.plug->o.key_ops, set_offset, &key, offset);
 	}
 
@@ -865,6 +866,7 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 
 	hint->len = 0;
 	hint->overhead = 0;
+	hint->insert_flags = 0;
 	unit_pos = place->pos.unit;
 	units = place->pos.unit == MAX_UINT32 ? 0 : extent40_units(place);
 
@@ -992,6 +994,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	uint64_t ins_off;
 	uint64_t uni_off;
 	uint64_t blk_off;
+	uint64_t total;
 	uint64_t start;
 	int64_t count;
 	int64_t bytes;
@@ -1027,7 +1030,8 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	max_off = plug_call(hint->maxkey.plug->o.key_ops,
 			    get_offset, &hint->maxkey);
 
-	count = max_off - ins_off;
+	total = max_off - ins_off;
+	count = total;
 
 	if (hint->insert_flags & ET40_OVERWRITE) {
 		/* If overwrite requires space, split the unit correctly. */
@@ -1138,7 +1142,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 			allocate = (start == EXTENT_UNALLOC_UNIT);
 		} else {
 			/* First unit or cannot merge with the last one. */
-			extent = extent40_body(place);
+			extent = extent40_body(place) + unit_pos;
 
 			/* If some data are being inserted, allocate some 
 			   blocks for them later. */
@@ -1169,11 +1173,13 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	/* Updating @key by unit key as it is changed. */
 	extent40_fetch_key(place, &key);
 
-	ins_off = plug_call(hint->offset.plug->o.key_ops,
-			    get_offset, &hint->offset);
-
 	extent = extent40_body(place) + place->pos.unit;
 	uni_off = plug_call(key.plug->o.key_ops, get_offset, &key);
+	
+	ins_off = plug_call(hint->offset.plug->o.key_ops,
+			    get_offset, &hint->offset);
+	
+	count = total;
 
 	/* Second stage -- writing data to allocated blocks. */
 	for ( ; count > 0; count -= size, ins_off += size) {
@@ -1229,7 +1235,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	}
 	
 	place_mkdirty(place);
-	return hint->count - count;
+	return total;
 }
 
 /* Calls @region_func for each block number extent points to. It is needed for
@@ -1259,7 +1265,7 @@ static errno_t extent40_layout(reiser4_place_t *place,
 		
 		width = et40_get_width(extent);
 		
-		if ((res = region_func(place, start, width, data)))
+		if ((res = region_func(start, width, data)))
 			return res;
 	}
 			
