@@ -269,15 +269,15 @@ static errno_t reg40_convert(object_entity_t *entity,
 {
 	errno_t res;
 	reg40_t *reg;
-	uint64_t fsize;
 	conv_hint_t hint;
 
 	aal_assert("umka-2467", plug != NULL);
 	aal_assert("umka-2466", entity != NULL);
 
 	reg = (reg40_t *)entity;
-	fsize = reg40_size(entity);
 
+	aal_memset(&hint, 0, sizeof(hint));
+	
 	/* Getting file data start key. We convert file starting from the zero
 	   offset until end is reached. */
 	plug_call(reg->position.plug->o.key_ops, assign,
@@ -287,11 +287,9 @@ static errno_t reg40_convert(object_entity_t *entity,
 		  &hint.offset, 0);
 	
 	/* Prepare convert hint. */
-	hint.chunk = 0;
-	hint.bytes = 0;
 	hint.plug = plug;
 
-	hint.count = fsize;
+	hint.count = MAX_UINT64;
 	hint.place_func = NULL;
 
 	/* Converting file data. */
@@ -299,7 +297,7 @@ static errno_t reg40_convert(object_entity_t *entity,
 		return res;
 
 	/* Updating stat data fields. */
-	return obj40_touch(&reg->obj, fsize, hint.bytes);
+	return obj40_touch(&reg->obj, MAX_UINT64, hint.bytes);
 }
 
 /* Make sure, that file body is of particular plugin type, that depends on tail
@@ -399,7 +397,7 @@ static int64_t reg40_cut(object_entity_t *entity, uint64_t offset) {
 		  set_offset, &hint.offset, offset);
 
 	/* Removing data from the tree. */
-	hint.count = size - offset;
+	hint.count = MAX_UINT64;
 	hint.shift_flags = SF_DEFAULT;
 	hint.data = reg->obj.info.tree;
 	hint.plug = reg40_policy_plug(reg, offset);
@@ -487,21 +485,24 @@ static errno_t reg40_truncate(object_entity_t *entity, uint64_t n) {
 			aal_error("Can't perform tail conversion.");
 			return res;
 		}
-		
-		/* Inserting holes. */
-		if ((bytes = reg40_put(entity, NULL, n - size, NULL)) < 0)
-			return bytes;
-		
+
 		/* Updating stat data fields. */
 		if ((res = obj40_update(&reg->obj)))
 			return res;
 		
-		bytes += obj40_get_bytes(&reg->obj);
-		
+		bytes = obj40_get_bytes(&reg->obj);
 		return obj40_touch(&reg->obj, n, bytes);
 	} else {
+		if (reg->body_plug->id.group == EXTENT_ITEM) {
+			uint32_t blksize;
+			
+			blksize = STAT_PLACE(&reg->obj)->node->block->size;
+			size = (n + blksize) / blksize * blksize;
+		} else
+			size = n;
+			
 		/* Cutting items/units */
-		if ((bytes = reg40_cut(entity, n)) < 0)
+		if ((bytes = reg40_cut(entity, size)) < 0)
 			return bytes;
 
 		/* Updating stat data fields. */
@@ -544,12 +545,9 @@ struct layout_hint {
 
 typedef struct layout_hint layout_hint_t;
 
-static errno_t cb_item_layout(void *place, blk_t start,
-			      count_t width, void *data)
-{
+static errno_t cb_item_layout(blk_t start, count_t width, void *data) {
 	layout_hint_t *hint = (layout_hint_t *)data;
-	return hint->region_func(hint->entity, start,
-				 width, hint->data);
+	return hint->region_func(start, width, hint->data);
 }
 
 /* Enumerates all blocks belong to file and calls passed @region_func for each
@@ -601,7 +599,7 @@ static errno_t reg40_layout(object_entity_t *entity,
 		} else {
 			blk_t blk = place_blknr(place);
 			
-			if ((res = cb_item_layout(place, blk, 1, &hint)))
+			if ((res = cb_item_layout(blk, 1, &hint)))
 				return res;
 		}
 
