@@ -3,7 +3,7 @@
    
    extents.c -- extents only tail policy plugin. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #include <reiser4/plugin.h>
 
 static int extents_tails(uint64_t value) {
@@ -18,7 +18,7 @@ static reiser4_plug_t extents_plug = {
 	.cl    = class_init,
 	.id    = {TAIL_NEVER_ID, 0, POLICY_PLUG_TYPE},
 	.label = "extents",
-	.desc  = "Tail policy \"extents only\" for reiser4, ver. " VERSION,
+	.desc  = "Tail policy \"extents only\" for reiser4. ",
 	.o = {
 		.policy_ops = &extents_ops
 	}
