@@ -21,7 +21,7 @@
 
 enum cpfs_behav_flags {
 	BF_FORCE      = 1 << 0,
-	BF_QUIET      = 1 << 1,
+	BF_YES        = 1 << 1,
 	BF_SHOW_PARM  = 1 << 2,
 	BF_SHOW_PLUG  = 1 << 3
 };
@@ -43,7 +43,7 @@ static void cpfs_print_usage(char *name) {
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
-		"  -q, --quiet                   forces acting without any questions.\n"
+		"  -y, --yes                     assumes an answer 'yes' to all questions.\n"
 		"  -f, --force                   makes cpfs to use whole disk, not\n"
 		"                                block device or mounted partition.\n"
 		"  -c, --cache N                 number of nodes in tree buffer cache\n");
@@ -81,7 +81,7 @@ int main(int argc, char *argv[]) {
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
 		{"force", no_argument, NULL, 'f'},
-		{"quiet", no_argument, NULL, 'q'},
+		{"yes", no_argument, NULL, 'y'},
 		{"print-profile", no_argument, NULL, 'p'},
 		{"print-plugins", no_argument, NULL, 'l'},
 		{"override", required_argument, NULL, 'o'},
@@ -93,7 +93,7 @@ int main(int argc, char *argv[]) {
 	memset(override, 0, sizeof(override));
 
 	/* Parsing parameters */    
-	while ((c = getopt_long(argc, argv, "hVqfplo:c:?", long_options, 
+	while ((c = getopt_long(argc, argv, "hVyfplo:c:?", long_options, 
 				(int *)0)) != EOF) 
 	{
 		switch (c) {
@@ -107,8 +107,8 @@ int main(int argc, char *argv[]) {
 		case 'f':
 			flags |= BF_FORCE;
 			break;
-		case 'q':
-			flags |= BF_QUIET;
+		case 'y':
+			flags |= BF_YES;
 			break;
 		case 'p':
 			flags |= BF_SHOW_PARM;
@@ -134,7 +134,7 @@ int main(int argc, char *argv[]) {
 		}
 	}
     
-	if (!(flags & BF_QUIET))
+	if (!(flags & BF_YES))
 		misc_print_banner(argv[0]);
 
 	/* Initializing libreiser4 (getting plugins, checking them on validness,
@@ -149,7 +149,7 @@ int main(int argc, char *argv[]) {
 	if (aal_strlen(override) > 0) {
 		override[aal_strlen(override) - 1] = '\0';
 
-		if (!(flags & BF_QUIET)) {
+		if (!(flags & BF_YES)) {
 			aal_mess("Overriding the plugin profile by \"%s\".",
 				 override);
 		}
@@ -263,8 +263,8 @@ int main(int argc, char *argv[]) {
 		goto error_free_src_device;
 	}
 
-	/* Checking for "quiet" mode */
-	if (!(flags & BF_QUIET)) {
+	/* Checking for non-interactive mode */
+	if (!(flags & BF_YES)) {
 		if (aal_yesno("All data on %s will be lost. Are you sure?",
 			      dst_dev) == EXCEPTION_OPT_NO)
 		{
@@ -316,7 +316,7 @@ int main(int argc, char *argv[]) {
 
 	if (reiser4_fs_copy(src_fs, dst_fs)) {
 		aal_error("Can't copy %s to %s.", src_dev, dst_dev);
-		goto error_free_dst_journal;
+		goto error_free_dst_fs;
 	}
 	
 	if (gauge) {
@@ -324,9 +324,6 @@ int main(int argc, char *argv[]) {
 		aal_gauge_free(gauge);
 	}
 
-	/* Freeing dst journal */
-	reiser4_journal_close(dst_fs->journal);
-
 	/* Closing dst fs */
 	reiser4_fs_close(dst_fs);
 
@@ -345,8 +342,6 @@ int main(int argc, char *argv[]) {
 	libreiser4_fini();
 	return NO_ERROR;
 	
- error_free_dst_journal:
-	reiser4_journal_close(dst_fs->journal);
  error_free_dst_fs:
 	reiser4_fs_close(dst_fs);
  error_free_src_fs:
