@@ -55,9 +55,8 @@ static void debugfs_print_usage(char *name) {
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
-		"  -q, --quiet                   forces using filesystem without\n"
-		"                                any questions.\n"
 		"  -f, --force                   makes debugfs to use whole disk, not\n"
+		"  -y, --yes                     assumes an answer 'yes' to all questions.\n"
 		"                                block device or mounted partition.\n"
 		"  -c, --cache N                 number of nodes in tree buffer cache\n"
 		"\n"
@@ -66,47 +65,35 @@ static void debugfs_print_usage(char *name) {
 }
 
 
-static errno_t cb_count_block(void *entity, blk_t start,
-			      count_t width, void *data) 
-{
+static errno_t cb_count_block(blk_t start, count_t width, void *data) {
 	(*(uint64_t *)data)++;
 	return 0;
 }
 
-static errno_t cb_mark_block(void *entity, blk_t start,
-			     count_t width, void *data)
-{
-	reiser4_fs_t *fs = (reiser4_fs_t *)entity;
+static errno_t cb_mark_block(blk_t start, count_t width, void *data) {
+	reiser4_fs_t *fs = (reiser4_fs_t *)data;
 	blk_t *backup = (blk_t *)fs->data;
-	uint64_t free;
+	errno_t res;
 
 	*backup = start;
 	backup++;
 	fs->data = backup;
 		
-	if (reiser4_alloc_occupied((reiser4_alloc_t *)data, start, width))
+	if (reiser4_alloc_occupied(fs->alloc, start, width))
 		return 0;
 		
-	if (!(free = reiser4_format_get_free(fs->format))) {
+	if ((res = reiser4_format_dec_free(fs->format, 1))) {
 		aal_error("No free blocks on the fs");
 		return -ENOSPC;
 	}
-
-	reiser4_format_set_free(fs->format, free - 1);
 	
-	return reiser4_alloc_occupy((reiser4_alloc_t *)data, start, width);
+	return reiser4_alloc_occupy(fs->alloc, start, width);
 }
 
-static errno_t cb_unmark_block(void *entity, blk_t start,
-			       count_t width, void *data)
-{
-	reiser4_fs_t *fs = (reiser4_fs_t *)entity;
-	uint64_t free;
-
-	free = reiser4_format_get_free(fs->format);
-	reiser4_format_set_free(fs->format, free + 1);
-	
-	return reiser4_alloc_release((reiser4_alloc_t *)data, start, width);
+static errno_t cb_unmark_block(blk_t start, count_t width, void *data) {
+	reiser4_fs_t *fs = (reiser4_fs_t *)data;
+	reiser4_format_inc_free(fs->format, 1);
+	return reiser4_alloc_release(fs->alloc, start, width);
 }
 
 /* Initializes exception streams used by debugfs */
@@ -154,7 +141,6 @@ static errno_t cb_reloc_node(reiser4_node_t *node, void *data) {
 	debugfs_backup_hint_t *backup;
 	reiser4_tree_t *tree;
 	ptr_hint_t ptr;
-	uint64_t free;
 	uint32_t pos;
 	errno_t res;
 	blk_t blk;
@@ -180,13 +166,13 @@ static errno_t cb_reloc_node(reiser4_node_t *node, void *data) {
 		{
 			aal_mess("Relocating the node %llu.", node->block->nr);
 
-			if (!(free = reiser4_format_get_free(tree->fs->format))) {
+			res = reiser4_format_dec_free(tree->fs->format, 1);
+			
+			if (res) {
 				aal_error("No free blocks on the fs");
 				return -EIO;
 			}
 
-			reiser4_format_set_free(tree->fs->format, free - 1);
-
 			blk = reiser4_fake_get();
 
 			if (reiser4_tree_get_root(tree) == node->block->nr)
@@ -212,15 +198,17 @@ static errno_t cb_reloc_node(reiser4_node_t *node, void *data) {
 }
 
 static errno_t cb_reloc_extent(reiser4_place_t *place, void *data) {
-	uint32_t units, blksize, i, pos;
-	uint64_t count, offset, free;
 	debugfs_backup_hint_t *backup;
+	uint64_t count, offset;
+	uint32_t blksize, i;
+	uint32_t units, pos;
 	reiser4_tree_t *tree;
 	reiser4_node_t *node;
 	aal_device_t *device;
 	trans_hint_t trans;
 	ptr_hint_t ptr;
 	int look = 0;
+	errno_t res;
 	blk_t blk;
 	
 	aal_assert("vpf-1691", place != NULL);
@@ -311,12 +299,10 @@ static errno_t cb_reloc_extent(reiser4_place_t *place, void *data) {
 			return -EIO;
 		}
 
-		if (!(free = reiser4_format_get_free(tree->fs->format))) {
+		if ((res = reiser4_format_dec_free(tree->fs->format, 1))) {
 			aal_error("No free blocks on the fs");
 			return -EIO;
 		}
-
-		reiser4_format_set_free(tree->fs->format, free - 1);
 	}
 	
 	/* Flush moved blocks on disk. */
@@ -360,7 +346,7 @@ int main(int argc, char *argv[]) {
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
 		{"force", no_argument, NULL, 'f'},
-		{"quiet", no_argument, NULL, 'q'},
+		{"yes", no_argument, NULL, 'y'},
 		{"cat", required_argument, NULL, 'k'},
 		{"print-tree", no_argument, NULL, 't'},
 		{"print-journal", no_argument, NULL, 'j'},
@@ -392,7 +378,7 @@ int main(int argc, char *argv[]) {
 	}
     
 	/* Parsing parameters */    
-	while ((c = getopt_long(argc, argv, "hVqftb:djk:n:i:o:plsaPUOFWB:c:C?",
+	while ((c = getopt_long(argc, argv, "hVyftb:djk:n:i:o:plsaPUOFWB:c:C?",
 				long_options, (int *)0)) != EOF) 
 	{
 		switch (c) {
@@ -453,8 +439,8 @@ int main(int argc, char *argv[]) {
 		case 'f':
 			behav_flags |= BF_FORCE;
 			break;
-		case 'q':
-			behav_flags |= BF_QUIET;
+		case 'y':
+			behav_flags |= BF_YES;
 			break;
 		case 'P':
 			behav_flags |= BF_PACK_META;
@@ -490,7 +476,7 @@ int main(int argc, char *argv[]) {
 		}
 	}
     
-	if (!(behav_flags & BF_QUIET))
+	if (!(behav_flags & BF_YES))
 		misc_print_banner(argv[0]);
 
 	if (libreiser4_init()) {
@@ -503,7 +489,7 @@ int main(int argc, char *argv[]) {
 	if (aal_strlen(override) > 0) {
 		override[aal_strlen(override) - 1] = '\0';
 		
-		if (!(behav_flags & BF_QUIET)) {
+		if (!(behav_flags & BF_YES)) {
 			aal_mess("Overriding the plugin profile by \"%s\".", 
 				 override);
 		}
@@ -548,6 +534,8 @@ int main(int argc, char *argv[]) {
 			
 	if (behav_flags & BF_UNPACK_META) {
 		aal_stream_t stream;
+		char buf[256];
+		int i;
 		
 		/* Prepare the bitmap if needed. */
 		if (bm_file) {
@@ -564,6 +552,34 @@ int main(int argc, char *argv[]) {
 		}
 		
 		aal_stream_init(&stream, stdin, &file_stream);
+		aal_stream_read(&stream, buf, 4);
+		
+		if (aal_strncmp(buf, VERSION_PACK_SIGN, 4)) {
+			aal_error("The metadata were packed with the "
+				  "reiser4progs version <= 1.0.2.");
+			goto error_free_bitmap;
+		}
+		
+		i = 0;
+		while (1) {
+			if (aal_stream_read(&stream, buf + i, 1) != 1) {
+				aal_error("Can't read from the stream. Is it over?");
+				goto error_free_bitmap;
+			}
+			
+			if (buf[i] == '\0' || i == 255)
+				break;
+			i++;
+		}
+
+		if (i == 255) {
+			aal_fatal("Can't detect the reiser4progs version "
+				  "the metadata were packed with.");
+			goto error_free_bitmap;
+		} else {
+			aal_info("The metadata were packed with the "
+				 "reiser4progs %s.", buf);
+		}
 		
 		if (!(fs = repair_fs_unpack(device, bitmap, &stream))) {
 			aal_error("Can't unpack filesystem.");
@@ -603,6 +619,12 @@ int main(int argc, char *argv[]) {
 		}
 
 		reiser4_opset_profile(fs->tree->ent.opset);
+
+		/* Opening the journal */
+		if (!(fs->journal = reiser4_journal_open(fs, device))) {
+			aal_error("Can't open journal on %s", host_dev);
+			goto error_free_fs;
+		}
 	}
 	
 	if (behav_flags & BF_PACK_META /* || print disk blocks */) {
@@ -650,21 +672,15 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	/* Opening the journal */
-	if (!(fs->journal = reiser4_journal_open(fs, device))) {
-		aal_error("Can't open journal on %s", host_dev);
-		goto error_free_bitmap;
-	}
-		
 	/* In the case no print flags was specified, debugfs will print super
 	   blocks by defaut. */
-	if (print_flags == 0 && (behav_flags & ~(BF_FORCE | BF_QUIET)) == 0)
+	if (print_flags == 0 && (behav_flags & ~(BF_FORCE | BF_YES)) == 0)
 		print_flags = PF_SUPER;
 
 	/* Handling print options */
 	if ((behav_flags & BF_CAT)) {
 		if (debugfs_browse(fs, cat_filename))
-			goto error_free_journal;
+			goto error_free_bitmap;
 	}
 	
 	if (print_flags & PF_SUPER) {
@@ -687,22 +703,24 @@ int main(int argc, char *argv[]) {
 
 	if (print_flags & PF_BLOCK) {
 		if (debugfs_print_block(fs, blocknr))
-			goto error_free_journal;
+			goto error_free_bitmap;
 	}
     
 	if (print_flags & PF_NODES || print_flags & PF_ITEMS) {
 		if (debugfs_print_file(fs, print_filename, print_flags))
-			goto error_free_journal;
+			goto error_free_bitmap;
 	}
 
 	if (behav_flags & BF_PACK_META) {
 		aal_stream_t stream;
 
 		aal_stream_init(&stream, stdout, &file_stream);
+		aal_stream_write(&stream, VERSION_PACK_SIGN, 4);
+		aal_stream_write(&stream, VERSION, sizeof(VERSION));
 		
 		if (repair_fs_pack(fs, bitmap, &stream)) {
 			aal_error("Can't pack filesystem.");
-			goto error_free_journal;
+			goto error_free_bitmap;
 		}
 		
 		aal_stream_fini(&stream);
@@ -722,11 +740,11 @@ int main(int argc, char *argv[]) {
 		
 		/* Mark all new backup blocks as used to not allocate them
 		   again on reallocation. */
-		if (reiser4_backup_layout(fs, cb_mark_block, fs->alloc)) {
+		if (reiser4_backup_layout(fs, cb_mark_block, fs)) {
 			aal_error("Failed to mark backup blocks used.");
 			aal_gauge_done(backup.gauge);
 			aal_gauge_free(backup.gauge);
-			goto error_free_journal;
+			goto error_free_fs;
 		}
 		
 		fs->data = NULL;
@@ -741,23 +759,20 @@ int main(int argc, char *argv[]) {
 			aal_error("Failed to free blocks for the new backup.");
 			aal_gauge_done(backup.gauge);
 			aal_gauge_free(backup.gauge);
-			goto error_free_journal;
+			goto error_free_fs;
 		}
 		aal_gauge_done(backup.gauge);
 		aal_gauge_free(backup.gauge);
 
 		/* Mark all old backup blocks as unused. */
-		if (reiser4_old_backup_layout(fs, cb_unmark_block, fs->alloc)) {
+		if (reiser4_old_backup_layout(fs, cb_unmark_block, fs)) {
 			aal_error("Failed to mark backup blocks used.");
-			goto error_free_journal;
+			goto error_free_fs;
 		}
 
 		aal_free(backup.blk);
 	}
 	
-	/* Closing the journal */
-	reiser4_journal_close(fs->journal);
-	
  done:
 	/* Closing filesystem itself */
 	reiser4_fs_close(fs);
@@ -770,8 +785,6 @@ int main(int argc, char *argv[]) {
 	libreiser4_fini();
 	return NO_ERROR;
 
- error_free_journal:
-	reiser4_journal_close(fs->journal);
  error_free_bitmap:
 	if (bitmap) {
 		aux_bitmap_close(bitmap);
