@@ -12,30 +12,6 @@
 #include <errno.h>
 #include "debugfs.h"
 
-/* Prints passed @buff into stdout. The special print function is needed because
-   we can't just put 4k buffer into stdout. */
-errno_t debugfs_print_buff(void *buff, uint32_t size) {
-	int len = size;
-	void *ptr = buff;
-
-	while (len > 0) {
-		int written;
-
-		if ((written = write(1, ptr, len)) <= 0) {
-			
-			if (errno == EINTR)
-				continue;
-			
-			return -EIO;
-		}
-		
-		ptr += written;
-		len -= written;
-	}
-
-	return 0;
-}
-
 errno_t debugfs_print_stream(aal_stream_t *stream) {
 	char buff[256];
 
@@ -52,17 +28,14 @@ errno_t debugfs_print_stream(aal_stream_t *stream) {
 		if ((size = aal_stream_read(stream, buff, size)) <= 0)
 			return size;
 		
-		if (debugfs_print_buff(buff, size))
-			return -EIO;
+		printf(buff);
 	}
 
 	return 0;
 }
 
 /* Prints passed @node. */
-static errno_t tprint_process_node(reiser4_tree_t *tree,
-				   reiser4_node_t *node, void *data)
-{
+static errno_t tprint_process_node(reiser4_node_t *node, void *data) {
 	aal_stream_t stream;
 
 	aal_stream_init(&stream, stdout, &file_stream);
@@ -75,7 +48,7 @@ static errno_t tprint_process_node(reiser4_tree_t *tree,
 }
 
 void debugfs_print_node(reiser4_node_t *node) {
-	tprint_process_node(NULL, node, NULL);
+	tprint_process_node(node, NULL);
 }
 
 /* Prints block denoted as blk */
@@ -175,7 +148,7 @@ void debugfs_print_status(reiser4_fs_t *fs) {
 void debugfs_print_format(reiser4_fs_t *fs) {
 	aal_stream_t stream;
 
-	if (!fs->format->entity->plug->o.format_ops->print) {
+	if (!fs->format->ent->plug->o.format_ops->print) {
 		aal_info("Format print method is not "
 			 "implemented.");
 		return;
@@ -192,7 +165,7 @@ void debugfs_print_format(reiser4_fs_t *fs) {
 void debugfs_print_oid(reiser4_fs_t *fs) {
 	aal_stream_t stream;
     
-	if (!fs->oid->entity->plug->o.oid_ops->print) {
+	if (!fs->oid->ent->plug->o.oid_ops->print) {
 		aal_info("Oid allocator print method is "
 			 "not implemented.");
 		return;
@@ -245,10 +218,10 @@ static errno_t fprint_process_place(
 {
 	fprint_hint_t *hint = (fprint_hint_t *)data;
 
-	if (node_blocknr(place->node) == hint->old)
+	if (place_blknr(place) == hint->old)
 		return 0;
 
-	hint->old = node_blocknr(place->node);
+	hint->old = place_blknr(place);
 	debugfs_print_node(place->node);
 
 	return 0;
@@ -264,7 +237,7 @@ errno_t debugfs_print_file(
 	fprint_hint_t hint;
 	reiser4_object_t *object;
 	
-	if (!(object = reiser4_object_open(fs->tree, filename, 0)))
+	if (!(object = reiser4_semantic_open(fs->tree, filename, NULL, 0)))
 		return -EINVAL;
 
 	/* If --print-file option is specified, we show only items belong to the
@@ -285,8 +258,10 @@ errno_t debugfs_print_file(
 
 		place_func = fprint_process_place;
 		
-		if ((res = reiser4_object_metadata(object, place_func, &hint))) {
-			aal_error("Can't print object %s metadata.", object->name);
+		if ((res = reiser4_object_metadata(object, place_func, &hint)))
+		{
+			aal_error("Can't print object %s metadata.", 
+				  reiser4_print_inode(&object->ent->object));
 		}
 	}
 
