@@ -9,6 +9,8 @@
 
 #include <fsck.h>
 
+#define fsck_opt(parse, flag) aal_test_bit(&(parse)->options, flag)
+
 static void fsck_print_usage(char *name) {
 	fprintf(stderr, "Usage: %s [ options ] FILE\n", name);
 
@@ -23,7 +25,7 @@ static void fsck_print_usage(char *name) {
 		"  -n, --no-log                  makes fsck to not complain\n"
 		"  -a, --auto                    automatically checks the consistency\n"
 		"                                without any questions.\n"
-		"  -v, --verbose                 makes fsck to be verbose\n"
+		"  -q, --quiet                   supresses gauges\n"
 		"  -r                            ignored\n"
 		"Plugins options:\n"
 		"  -p, --print-profile           prints the plugin profile.\n"
@@ -34,7 +36,7 @@ static void fsck_print_usage(char *name) {
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
-		"  -q, --quiet                   supresses progress information.\n"
+		"  -y, --yes                     assumes an answer 'yes' to all questions.\n"
 		"  -f, --force                   makes fsck to use whole disk, not block\n"
 		"                                device or mounted partition.\n"
 		"  -c, --cache N                 number of nodes in tree buffer cache\n");
@@ -85,8 +87,10 @@ static errno_t fsck_ask_confirmation(fsck_parse_t *data, char *host_name) {
 		break;
 	}
 	
-	if (aal_yesno("Continue?") == EXCEPTION_OPT_NO) 
-		return USER_ERROR;
+	if (!fsck_opt(data, FSCK_OPT_YES)) {
+		if (aal_yesno("Continue?") == EXCEPTION_OPT_NO) 
+			return USER_ERROR;
+	}
      
 	return NO_ERROR; 
 }
@@ -132,16 +136,17 @@ static errno_t fsck_init(fsck_parse_t *data,
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
 		{"quiet", no_argument, NULL, 'q'},
+		{"yes", no_argument, NULL, 'y'},
 		{"no-log", no_argument, NULL, 'n'},
 		{"auto", no_argument, NULL, 'a'},
 		{"force", no_argument, NULL, 'f'},
-		{"verbose", no_argument, NULL, 'v'},
 		{"cashe", required_argument, 0, 'c'},
 		{"override", required_argument, NULL, 'o'},
 		/* Fsck hidden options. */
 		{"passes-dump", required_argument, 0, 'U'},
 		{"backup", required_argument, 0, 'b'},
 		{"bitmap", required_argument, 0, 'B'},
+		{"unused", no_argument, NULL, 'u'},
 		{0, 0, 0, 0}
 	};
 
@@ -155,7 +160,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 		return USER_ERROR;
 	}
 
-	while ((c = getopt_long(argc, argv, "L:VhnqafvU:b:r?dB:plo:c:", 
+	while ((c = getopt_long(argc, argv, "L:VhnqafU:b:r?dB:plo:c:uy", 
 				options, &option_index)) >= 0) 
 	{
 		switch (c) {
@@ -186,9 +191,6 @@ static errno_t fsck_init(fsck_parse_t *data,
 		case 'a':
 			aal_set_bit(&data->options, FSCK_OPT_AUTO);
 			break;
-		case 'v':
-			aal_set_bit(&data->options, FSCK_OPT_VERBOSE);
-			break;
 		case 'h': 
 		case '?':
 			fsck_print_usage(argv[0]);
@@ -199,6 +201,9 @@ static errno_t fsck_init(fsck_parse_t *data,
 		case 'q':
 			aux_gauge_set_handler(NULL, GT_PROGRESS);
 			break;
+		case 'y':
+			aal_set_bit(&data->options, FSCK_OPT_YES);
+			break;
 		case 'r':
 			break;
 		case 'd':
@@ -226,6 +231,9 @@ static errno_t fsck_init(fsck_parse_t *data,
 			aal_strncat(override, optarg, aal_strlen(optarg));
 			aal_strncat(override, ",", 1);
 			break;
+		case 'u':
+			aal_set_bit(&data->options, FSCK_OPT_WHOLE);
+			break;
 		}
 	}
 	
@@ -437,34 +445,48 @@ static errno_t fsck_check_init(repair_data_t *repair,
 
 static errno_t fsck_check_fini(repair_data_t *repair) {
 	reiser4_status_t *status;
-	uint64_t state = 0;
+	aal_device_t *device;
+	uint64_t state;
 	int flags;
 
 	aal_assert("vpf-1338", repair != NULL);
 	aal_assert("vpf-1339", repair->fs != NULL);
 	aal_assert("vpf-1340", repair->fs->status != NULL);
 	
-	if (repair->mode == RM_CHECK)
-		return 0;
+	device = repair->fs->device;
+	flags = device->flags;
 	
-	/* Fix the status block. */
-	if (repair->fatal)
-		state = FS_DAMAGED;
-	else if (repair->fixable)
-		state = FS_CORRUPTED;
-	
-	status = repair->fs->status;
-	flags = status->device->flags;
-	
-	repair_status_state(status, state);
-	
-	if (aal_device_reopen(status->device, status->device->blksize, O_RDWR))
+	if (aal_device_reopen(device, device->blksize, O_RDWR))
 		return -EIO;
 	
-	if (reiser4_status_sync(repair->fs->status))
-		return -EIO;
+	if (repair->mode == RM_CHECK) {
+		/* Fix the status block. */
+		status = repair->fs->status;
+		
+		if (repair->fatal)
+			state = FS_DAMAGED;
+		else if (repair->fixable)
+			state = FS_CORRUPTED;
+		else
+			state = 0;
 
-	if (aal_device_reopen(status->device, status->device->blksize, flags))
+		repair_status_state(status, state);
+	
+		if (reiser4_status_sync(repair->fs->status))
+			return -EIO;
+	}
+
+	/* If there was no backup openned or some fields have been changed, 
+	   reopen the backup. */
+	if (!(repair->fs->backup = repair_backup_reopen(repair->fs))) {
+		aal_fatal("Failed to reopen backup.");
+		return -EIO;
+	}
+
+	if (reiser4_backup_sync(repair->fs->backup))
+		return -EIO;
+	
+	if (aal_device_reopen(device, device->blksize, flags))
 		return -EIO;
 
 	return 0;
@@ -503,7 +525,11 @@ int main(int argc, char *argv[]) {
 	fsck_time("fsck.reiser4 started at");
 	
 	/* SB_mode is specified, otherwise  */
-	repair.debug_flag = aal_test_bit(&parse_data.options, FSCK_OPT_DEBUG);
+	repair.flags = 
+		fsck_opt(&parse_data, FSCK_OPT_DEBUG) << REPAIR_DEBUG |
+		fsck_opt(&parse_data, FSCK_OPT_WHOLE) << REPAIR_WHOLE |
+		fsck_opt(&parse_data, FSCK_OPT_YES) << REPAIR_YES;
+		
 	repair.bitmap_file = parse_data.bitmap_file;
 	
 	res = fsck_check_init(&repair, device, parse_data.backup, 
@@ -514,14 +540,15 @@ int main(int argc, char *argv[]) {
 		
 	stage = 1;
 	
-	if ((res = repair_check(&repair)) || (res = fsck_check_fini(&repair)))
-		goto free_fs;
+	res = repair_check(&repair);
 
+	/* Even if there was some problems on fs check, fini must be done. */
+	res |= fsck_check_fini(&repair);
+	
 	fsck_time("fsck.reiser4 finished at");
     
- free_fs:
 	fprintf(stderr, "Closing fs...");
-	repair_fs_close(repair.fs);
+	reiser4_fs_close(repair.fs);
 	repair.fs = NULL;
 	fprintf(stderr, "done\n");
     
@@ -561,7 +588,7 @@ int main(int argc, char *argv[]) {
 
 	if (repair.fatal) {
 		/* Some fatal corruptions in disk format or filesystem. */
-		if (parse_data.fs_mode == RM_BUILD) {
+		if (parse_data.fs_mode == RM_BUILD && stage) {
 			/* Only if no metadata are found. */
 			fprintf(stderr, "NO REISER4 METADATA WERE FOUND. "
 				"FS RECOVERY IS NOT POSSIBLE.\n");
@@ -581,7 +608,7 @@ int main(int argc, char *argv[]) {
 		ex = FIXABLE_ERROR;
 	} else if (!repair.sb_fixable) {
 		if (parse_data.fs_mode != RM_CHECK && 
-		    aal_test_bit(&parse_data.options, FSCK_OPT_RO))
+		    fsck_opt(&parse_data, FSCK_OPT_RO))
 		{
 			fprintf(stderr, "\nThe filesystem was mounted RO. It is "
 				"better to umount and mount it again.\n");
